# encoding=utf-8

import struct

HEADER_SIZE = 4


class SimpleProtocolFilter(object):
    """ 简单协议过滤器
        协议按照 [有效数据字节数][有效数据] 这种协议包的格式进行打包和解包
        [有效数据字节数]长度HEADER_SIZE字节
        [有效数据]长度有效数据字节数字节
        本类按照这种方式，顺序从数据流中取出数据进行拼接，一旦接收完一个完整的协议包，就会将协议包返回
        [有效数据]字段接收到后会按照utf-8进行解码，因为在传输过程中是用utf-8进行编码的
        所有编解码的操作在该类中完成
    """

    def __init__(self):
        super(SimpleProtocolFilter, self).__init__()
        self.buf = ''

    def input(self, data):
        """ 小数据片段拼接成完整数据包
            如果内容足够则yield数据包
        """
        self.buf += data
        while len(self.buf) > HEADER_SIZE:
            data_len = struct.unpack('i', self.buf[0:HEADER_SIZE])[0]
            if len(self.buf) >= data_len + HEADER_SIZE:
                content = self.buf[HEADER_SIZE:data_len + HEADER_SIZE]
                self.buf = self.buf[data_len + HEADER_SIZE:]
                yield content
            else:
                break

    @staticmethod
    def pack(content):
        """ content should be str
        """
        return struct.pack('i', len(content)) + content

    @staticmethod
    def unpack(data):
        """ return length, content
        """
        length = struct.unpack('i', data[0:HEADER_SIZE])
        return length[0], data[HEADER_SIZE:]


if __name__ == '__main__':
    s = SimpleProtocolFilter()
    r = s.pack("nimei")
    print(repr(r))
    u = s.unpack(r)
    g = s.input(r[:1])
    for i in g:
        print i
    g = s.input(r[1:])
    for i in g:
        print i



# encoding=utf-8
from simplerpc import RpcAgent
import simplerpc
import time



class RpcClient(RpcAgent):

    CONNECTING, CONNECTED, CLOSED = 1, 2, 3

    """docstring for RpcClient"""
    def __init__(self, conn):
        super(RpcClient, self).__init__()
        self.conn = conn
        self.conn.connect_cb = self.on_connect
        self.conn.close_cb = self.on_close
        self._status = self.CONNECTING
        self.conn.connect()

    @property
    def DEBUG(self):
        return simplerpc.DEBUG

    @DEBUG.setter
    def DEBUG(self, value):
        simplerpc.DEBUG = value

    def on_connect(self):
        print("on_connect")
        if self._status == self.CONNECTING:
            self._status = self.CONNECTED

    def on_close(self):
        print("on_close")
        self._status = self.CLOSED

    def call(self, func, *args, **kwargs):
        msg, cb = self.format_request(func, *args, **kwargs)
        self.conn.send(msg)
        return cb

    def update(self):
        if self._status != self.CONNECTED:
            return
        data = self.conn.recv()
        if not data:
            return
        for msg in data:
            self.handle_message(msg, self.conn)

    def wait_connected(self):
        for i in range(10):
            print("waiting for connection...%s" % i)
            if self._status == self.CONNECTED:
                return True
            elif self._status == self.CONNECTING:
                time.sleep(0.5)
            else:
                raise RuntimeError("Connection Closed")
        raise RuntimeError("connecting timeout")



# coding=utf-8


from pocounit.case import PocoTestCase
from pocounit.addons.poco.action_tracking import ActionTracker

from poco.drivers.unity3d import UnityPoco


class TutorialCase(PocoTestCase):
    @classmethod
    def setUpClass(cls):
        cls.poco = UnityPoco(('10.254.44.76', 5001))
        action_tracker = ActionTracker(cls.poco)
        cls.register_addin(action_tracker)



# coding=utf-8

import unittest

from hunter_cli import Hunter, open_platform
from poco.drivers.netease.mh import MhPoco


class TestBag(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        ipaddr = "10.254.46.45"
        tokenid = open_platform.get_api_token('poco on android')
        cls.poco = MhPoco(addr=(ipaddr, 5001))
        cls.hunter = Hunter(tokenid, 'mh', devip=ipaddr)

    def open_bag(self):
        max_times = 3
        count = 0
        while not self.poco('main/道具').exists():
            self.poco('main/切换按钮动画r').focus('center').click()
            count += 1
            if count >= max_times:
                raise Exception("bag not found")

        self.poco('main/道具').sibling('main/按钮底托').focus('center').click()
        self.poco(type='Button', textMatches='^道.*具$').focus('center').click()

    def server_call(self, cmd):
        self.hunter.script(cmd, lang='text')

    def setUp(self):
        self.server_call('$pckt di')
        self.server_call('$at set ilearncash 0')
        self.server_call('$at set icash 20000000')
        self.server_call('$at set $id iPcktPage 0')
        self.server_call('$at call_txe state_change($id,PROP_PCKTPAGE)')
        self.server_call('$at unlock_all')

        self.open_bag()

    def runTest(self):
        self.poco(type='Button', textMatches='^行.*囊$').focus('center').click()

        # 扩展行囊
        self.poco(text='+').focus('center').click()
        self.poco(text='确定').focus('center').click()
        self.poco(text='确定').focus('center').click()

        moneyCols = self.poco('heroitem_msg.VPanelEquip.HeroItem_Panel_EquipM_UI').child(type='VEdit')
        self.assertTrue(
            all([n.get_text() == '0' for n in moneyCols]),
            "行囊扩展成功"
        )

    def tearDown(self):
        self.poco('vstyle/button/按钮-关闭').focus([0.5, 0]).click()


if __name__ == '__main__':
    unittest.main()



import sys
sys.path.append("../../..")
from simplerpc.rpcclient import RpcClient
from time import sleep

from pprint import pprint


def test_with_tcp():
    from simplerpc.transport.tcp import TcpClient

    client = TcpClient()
    c = RpcClient(client)
    c.run(backend=True)
    c.wait_connected()
    test_client(c)

def test_with_sszmq():
    from simplerpc.transport.sszmq import SSZmqClient

    client = SSZmqClient()
    c = RpcClient(client)
    c.run(backend=True)
    c.wait_connected()
    test_client(c)


def test_client(c):
    """
    # unity rpc test
    print c.call("Add", 1, 2).wait()
    print(222)
    print c.call("Screen", 1, 2).wait()
    j, e = c.call("Dump", 1, 2).wait()
    print json.dumps(j)
    """
    # simply call rpc
    c.call("foobar", foo="aaa", bar="bbb")
    # call rpc and wait for rpc result
    cb = c.call("foo", foo=1, bar=2)
    r = cb.wait()
    print("wait and got:", r)
    cb = c.call("make_error")
    r = cb.wait()
    print("wait and got:", r)
    # call rpc and register callback
    c.call("foobar", foo="aaa", bar="bbb").on_result(pprint)
    # call rpc and register callback, then wait for rpc result
    cb = c.call("foobar2", foo="aaa", bar="bbb")
    cb.on_error(pprint)
    print("after callback, wait and got error:", cb.wait())
    # call rpc and wait for delay rpc result
    cb = c.call("delayecho", 111, 222)
    cb.on_result(pprint)
    print("after callback, wait and got delayed result: ", cb.wait())

    cb = c.call("delayerror", 111, 222)
    cb.on_result(pprint)
    print("after callback, wait and got delayed result: ", cb.wait())
    # run python console
    # c.console_run({"c": c})


if __name__ == '__main__':
    test_with_tcp()
    # test_with_sszmq()



# encoding=utf-8
import sys
sys.path.append("../../..")
from simplerpc.ssrpc.plugin import PluginRepo, Plugin, SSRpcClient, AgentManager, RemotePlugin
from pprint import pprint


def test_client(c, c2):
    # simply call rpc
    print c.call("foobar", foo="aaa", bar="bbb").wait()


    RemoteAAA = RemotePlugin(AAAPlugin.UUID)
    print RemoteAAA.minus(555, 1).wait()
    print AgentManager.REPO

    print RemoteAAA._all().minus(555, 111)
    cbs = RemoteAAA._all("SERVER").minus(555, 222)
    for cb in cbs:
        print cb.wait(), cb.agent

    print RemoteAAA._all("SERVER2").minus(555, 333)

    AgentManager.set_main_agent(c2)
    cb = RemoteAAA.minus(555, 444)
    print cb.wait(), cb.agent

    AgentManager.set_main_agent(c)
    cb = RemoteAAA.minus(555, 444)
    print cb.wait(), cb.agent


class AAAPlugin(Plugin):

    UUID = "AAAAAAA"

    def _on_rpc_ready(self, agent):
        agent.get_plugin(self.UUID).minus(3, 5).on_result(pprint)
        agent.get_plugin(self.UUID).add(3, 5).on_error(pprint)
        agent.get_plugin("BBBBBBB").echo(3, 5).on_result(pprint)

    def add(self, a, b):
        return a + b


def test_ssrpc():
    from simplerpc.transport.tcp import TcpClient
    PluginRepo.register(AAAPlugin())
    AgentManager.ROLE = "CLIENT"

    client = TcpClient()
    c = SSRpcClient(client)
    # c.console_run({"c": c})
    c.run(backend=True)
    c.wait_connected()

    client2 = TcpClient()
    c2 = SSRpcClient(client2)
    c2.run(backend=True)
    c2.wait_connected()
    # #
    import time
    time.sleep(2)
    test_client(c, c2)
    time.sleep(200)


if __name__ == '__main__':
    test_ssrpc()



# coding=utf-8

import base64
import time
import json

from poco.drivers.unity3d import UnityPoco
from poco.drivers.unity3d.unity3d_poco import DEFAULT_ADDR, Dumper
from poco.utils.simplerpc.rpcclient import RpcClient
from poco.utils.simplerpc.transport.tcp import TcpClient


def dump():
    conn = TcpClient(DEFAULT_ADDR)
    c = RpcClient(conn)
    c.DEBUG = False
    c.run(backend=True)
    time.sleep(2)
    t0 = time.time()
    d = Dumper(c)
    h = d.dumpHierarchy()
    t1 = time.time()
    print(t1 - t0)
    return h


def test_click():
    p = UnityPoco(('10.254.46.45', 5001))
    for star in p('star'):
        star.click()


def test_drag():
    p = UnityPoco(('10.254.46.45', 5001))
    shell = p('shell')
    for star in p('star'):
        star.drag_to(shell)


if __name__ == '__main__':
    # test_click()
    # test_drag()
    # time.sleep(2)
    #
    p = UnityPoco(("localhost", 5001), unity_editor=True)
    p('pos_input').set_text('2333')
    time.sleep(2)




# coding=utf-8

import base64
import os
from airtest.cli.runner import device as current_device
try:
    from airtest.core.api import snapshot
except ImportError:
    # 兼容旧版本
    from airtest.core.main import snapshot
from airtest.core.settings import Settings
from poco.sdk.interfaces.screen import ScreenInterface


class AirtestScreen(ScreenInterface):
    def __init__(self):
        super(AirtestScreen, self).__init__()

    def getPortSize(self):
        disp = current_device().display_info
        if disp['orientation'] in (1, 3):
            return [disp['height'], disp['width']]
        else:
            return [disp['width'], disp['height']]

    def getScreen(self, width):
        filename = 'poco-screenshot.png'
        screen_dir = os.path.join(Settings.LOG_DIR, Settings.SCREEN_DIR)
        if not os.path.exists(screen_dir):
            os.makedirs(screen_dir)
        filepath = os.path.join(screen_dir, filename)
        snapshot(filename)
        return base64.b64encode(open(filepath, 'rb').read()), 'png'



# coding=utf-8
__author__ = 'lxn3032'


import functools


def retries_when(exctypes, count=3):
    def wrapper(func):
        @functools.wraps(func)
        def wrapped(*args, **kwargs):
            ex = None
            for i in range(count):
                try:
                    return func(*args, **kwargs)
                except exctypes as e:
                    ex = e
            if ex:
                raise ex
        return wrapped
    return wrapper



# coding=utf-8
from __future__ import unicode_literals

import time
import warnings

from poco.exceptions import PocoTargetTimeout

__author__ = 'lxn3032'


class PocoAccelerationMixin(object):
    """
    This class provides some high-level method to reduce redundant code implementations.
    As this is a MixinClass, please do not introduce new state in methods.
    """

    def dismiss(self, targets, exit_when=None, sleep_interval=0.5, appearance_timeout=20, timeout=120):
        """
        Automatically dismiss the target objects

        Args:
            targets (:obj:`list`): list of poco objects to be dropped
            exit_when: termination condition, default is None which means to automatically exit when list of
             ``targets`` is empty
            sleep_interval: time interval between each actions for the given targets, default is 0.5s
            appearance_timeout: time interval to wait for given target to appear on the screen, automatically exit when
             timeout, default is 20s
            timeout: dismiss function timeout, default is 120s

        Raises:
            PocoTargetTimeout: when dismiss time interval timeout, under normal circumstances, this should not happen
             and if happens, it will be reported
        """

        try:
            self.wait_for_any(targets, timeout=appearance_timeout)
        except PocoTargetTimeout:
            # here returns only when timeout
            # 仅当超时时自动退出
            warnings.warn('Waiting timeout when trying to dismiss something before them appear. Targets are {}'
                          .encode('utf-8').format(targets))
            return

        start_time = time.time()
        while True:
            no_target = True
            for t in targets:
                if t.exists():
                    try:
                        for n in t:
                            try:
                                n.click(sleep_interval=sleep_interval)
                                no_target = False
                            except:
                                pass
                    except:
                        # Catch the NodeHasBeenRemoved exception if some node was removed over the above iteration
                        # and just ignore as this will not affect the result.
                        # 遍历(__iter__: for n in t)过程中如果节点正好被移除了，可能会报远程节点被移除的异常
                        # 这个报错忽略就行
                        pass
            time.sleep(sleep_interval)
            should_exit = exit_when() if exit_when else False
            if no_target or should_exit:
                return

            if time.time() - start_time > timeout:
                raise PocoTargetTimeout('dismiss', repr(targets).decode('utf-8'))



# -*- coding: utf-8 -*-
"""---------------------------------------------------------------------
Author:
guoxiaocan
gzguoxiaocan@corp.netease.com

Date:
2017/2/13

Description:

History:
2017/2/13, create file.
---------------------------------------------------------------------"""
import sys
sys.path.append("..")

import time
from SSRPC import MAIN_THREAD_TICK
from SSRPC import SERV_MODE_ALL_GET_MSG
from SSRPC import SSRPCCliHandler
from SSRPC import SSRPCSvrHandler
from SSRPC import init
from SSRPC import setup_client
from SSRPC import setup_server
from SSRPC import ssrpc_method
from SSRPC import tick


class SH(SSRPCSvrHandler):
	def on_init_finish(self, rtn):
		print(self._my_id, "init ok")

	def on_data(self, client_id, data):
		print(self._my_id, "server from", client_id, "on_data", data)
		self.send(client_id, "efg")

	@ssrpc_method()
	def foo(self, client_id, data):
		print(self._my_id, "server from", client_id, "rpc_func foo", data)
		self.SSRPC[client_id].bar("efg")

	@ssrpc_method()
	def foo2(self, client_id, data):
		print(self._my_id, "server from", client_id, "rpc_func foo", data)
		return "ijk", "lmn"


class CH(SSRPCCliHandler):
	def __init__(self):
		super(SSRPCCliHandler, self).__init__()
		self.count = 0

	def _callback(self, x, y):
		print("callback", x, y)
		self.my_send()

	def my_send(self):
		self.count += 1
		if self.count > 10:
			return
		elif self.count > 8:
			rtn = self.SSRPC.foo2({"ghi":self.count}).wait()
			print("sync call", rtn)
			self.my_send()
		elif self.count > 6:
			self.SSRPC.foo2({"def":self.count}).callback(self._callback)
		elif self.count > 3:
			self.SSRPC.foo({"def":self.count})
		else:
			self.send({"abc":self.count , "cdf":"abc"*1024})

	def on_init_finish(self, rtn):
		print(self._my_id, "SSRPCCliHandler init ok")
		self.my_send()

	def on_data(self, data):
		print(self._my_id, "client on_data", data)
		self.my_send()

	@ssrpc_method()
	def bar(self, test_str):
		print(self._my_id, "client rpc_func bar", test_str)
		self.my_send()

def my_err_handler(code, msg):
	print("my_err_handler %d %s"%(code, msg))

init("g68", MAIN_THREAD_TICK, my_err_handler)

def server_test():
	print("setup_server")
	setup_server("ff_server", SH(), SERV_MODE_ALL_GET_MSG)
	setup_server(["server1","ff_server"], SH(), SERV_MODE_ALL_GET_MSG)

	while True:
		tick()
		time.sleep(0.1)

from threading import Thread
p = Thread(target=server_test)
p.daemon = True
p.start()

time.sleep(1)

# setup_client("ff_server", CH())
# setup_client("ff_server", CH())
clientHandler = CH()
setup_client("ff_server", clientHandler)

import time
while True:
	tick()
	time.sleep(0.1)




# -*- coding: utf-8 -*-
"""----------------------------------------------------------------------------
Author:
	fengfan
	fengfan@corp.netease.com
Date:
	2017/1/10
Description:
	转发的中心节点
History:
	2017/1/10, create file.
----------------------------------------------------------------------------"""
import random
import threading
import time
import os
import sys
import zmq


BIND_IP = "0.0.0.0"  # 小小云不能bind外服ip，所以直接bind 0.0.0.0
IP = "59.111.129.112"  # 小小云外服IP
PORT = 5560

LOG_FILE = "./broker_connect.log"  # 现在日志都打在一起


# from .SSRPC import CMD_CREATE_SERVER, CMD_CREATE_CLIENT, CMD_SERVER_QUIT, _pack, _unpack, SERV_MODE_ALL_GET_MSG
CMD_CREATE_SERVER = 1
CMD_CREATE_CLIENT = 2
CMD_SERVER_QUIT = 3
CMD_GET_SERVER_NODE = 4

SERV_MODE_ALL_GET_MSG = 0x1
import msgpack
_pack = lambda dat: msgpack.packb(dat, use_bin_type=True)
_unpack = lambda dat: msgpack.unpackb(dat, encoding="utf-8")

def match_key(projectID, serverID):
	return (projectID, serverID)  # 用tuple存，为了查询用

_HEARTBEAT_SEC = 5
_HEARTBEAT_DATA = b'_ss_inner_heartbeat'
def new_pair(router_port, dealer_port, server_id, mode, log_file):
	import logging
	log = logging.getLogger(server_id)
	logFH = logging.FileHandler(log_file)
	logFH.setFormatter(logging.Formatter('%(asctime)s - %(filename)s - [line:%(lineno)d] - %(levelname)s - %(message)s'))
	log.addHandler(logFH)
	log.setLevel(logging.INFO)

	ctx = zmq.Context()
	client_router = ctx.socket(zmq.ROUTER)
	server_router = ctx.socket(zmq.ROUTER)
	client_router.bind("tcp://%s:%d"%(BIND_IP, router_port))
	server_router.bind("tcp://%s:%d"%(BIND_IP, dealer_port))

	log.info("pair process begin, router:%d <-> dealer:%d", router_port, dealer_port)

	server_ids = {}
	client_server_map = {}

	def hex_str(tstr):
		return ":".join(hex(x if isinstance(x, int) else ord(x))[2:] for x in tstr)

	def clear_client_server_map():
		now = time.time()
		for k in list(client_server_map):
			if now - client_server_map[k][1] > _HEARTBEAT_SEC*2:
				client_server_map.pop(k)
				log.info("client %s disconnect", hex_str(k))

	def clear_server_ids():
		now = time.time()
		for m in list(server_ids):
			if now - server_ids[m] > _HEARTBEAT_SEC*2:
				server_ids.pop(m)

	poller = zmq.Poller()
	poller.register(client_router, zmq.POLLIN)
	poller.register(server_router, zmq.POLLIN)
	while True:
		socks = dict(poller.poll(_HEARTBEAT_SEC/2*1000))

		if socks.get(client_router) == zmq.POLLIN:
			m = client_router.recv_multipart()
			# log.debug("client %s got message (%d) %s, %d"%(hex_str(m[0]), len(m), m[1], len(server_ids)))
			for i in range(0, len(m), 2):
				minfo = client_server_map.get(m[i])
				if not minfo:
					if len(server_ids) == 0: # TODO: 还没有server
						continue
					if mode & SERV_MODE_ALL_GET_MSG:
						minfo = ["\x0a\x11", 0]
					else:
						minfo = [random.choice(list(server_ids.keys())), 0]

					client_server_map[m[i]] = minfo
					log.info("client %s connect to %s", hex_str(m[i]), hex_str(minfo[0]))

				minfo[1] = time.time()
				# client heartbeat sends to server by f.f. 2017-02-22
				# if m[i+1] == _HEARTBEAT_DATA:
				# 	continue

				if minfo[0] is "\x0a\x11":  # all servers get msg, broadcast it
					for sid in server_ids:
						server_router.send_multipart([sid, m[i], m[i+1]])
					log.info("client:%s sends msg to all", hex_str(m[i]))
				else:
					server_router.send_multipart([minfo[0], m[i], m[i+1]])
					log.info("client:%s sends msg to %s", hex_str(m[i]), hex_str(minfo[0]))

		if socks.get(server_router) == zmq.POLLIN:
			m = server_router.recv_multipart()
			# log.debug("server %s got message (%d)"%(hex_str(m[0]), len(m)))
			for i in range(0, len(m), 3):
				# server send msg to me
				server_id = m[i]
				if m[i+1] == b"imsvr":  # reg server protocol
					server_ids[server_id] = time.time()
					log.info("server %s connect as %s", server_id, hex_str(m[i]))
					continue
				elif m[i+1] == _HEARTBEAT_DATA:  # server heart beat
					server_ids[server_id] = time.time()
					continue
				elif m[i+1] == b"myclients":  # get all clients in this node
					server_router.send_multipart([server_id, b"myclients_cb", _pack(client_server_map)])
					continue

				# server send msg to client
				client_router.send_multipart([m[i+1],m[i+2]])
				log.info("server:%s sends msg to %s", hex_str(m[i]), hex_str(m[i+1]))

		# 清理过期链接
		clear_client_server_map()
		old_server_cnt = len(server_ids)
		clear_server_ids()
		if old_server_cnt != 0 and len(server_ids) == 0:  # 可以下线了
			break

	client_router.close()
	server_router.close()
	ctx.term()
	log.info("pair process end, router:%d <-> dealer:%d", router_port, dealer_port)


def get_available_port_pair():
	import socket
	s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	s.bind(("", 0))
	s.listen(1)
	sport = s.getsockname()[1]

	b = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	b.bind(("", 0))
	b.listen(1)
	bport = b.getsockname()[1]

	s.close()
	b.close()
	print("new pair socket create, router:%d <-> dealer:%d"%(sport, bport))
	return sport, bport

SERVER_INFO = {}
PROCESS_INFO = {}


def _send_msg(sock, data):
	sock.send(_pack(data))


def _check_sub_process_quit(sock_pairs, p):
	global SERVER_INFO
	global PROCESS_INFO
	p.join()
	PROCESS_INFO.pop(sock_pairs)
	for key in list(SERVER_INFO):
		if SERVER_INFO[key] == sock_pairs:
			SERVER_INFO.pop(key)
	print("socket_pair for %s terminated"%(str(sock_pairs)))


def on_create_server(sock, msg):
	# TODO: project info seperate
	global SERVER_INFO
	global PROCESS_INFO
	serv_ids = msg["server_ids"]

	for idt in serv_ids: # 检查一下是否已经有了对应的pair
		key = match_key(msg["proj"], idt)
		if key in SERVER_INFO:
			sock_pairs = SERVER_INFO[key]
			break # 有同名的
	else: # 没有
		sock_pairs = get_available_port_pair()
		from multiprocessing import Process
		p = Process(target=new_pair, args=list(sock_pairs)+[serv_ids[0], msg.get("mode", 0), LOG_FILE])
		p.daemon = True
		p.start()
		PROCESS_INFO[sock_pairs] = p
		threading.Thread(target=_check_sub_process_quit, name="%s_check"%(str(sock_pairs)), args=(sock_pairs, p)).start()
		print("%s: %s server set to pairs:%s, pid:%d"%(msg["proj"], serv_ids, sock_pairs, p.pid))

	for idt in serv_ids:
		key = match_key(msg["proj"], idt)
		if key in SERVER_INFO and SERVER_INFO[key] != sock_pairs:
			print("server:%s already has pairs:%s"%(key, SERVER_INFO[key]))
		SERVER_INFO[key] = sock_pairs

	msg[u"rtn"] = u"tcp://%s:%d"%(IP, sock_pairs[1])
	_send_msg(sock, msg)


def on_server_quit(sock, msg):
	global SERVER_INFO
	global PROCESS_INFO

	key = match_key(msg["proj"], msg["server_id"])
	pair = SERVER_INFO.get(key)
	if not pair:
		print("no such key: %s found"%(key))
		return

	finish_servers = []
	for k in SERVER_INFO.keys():
		if SERVER_INFO[k] == pair:
			SERVER_INFO.pop(k)
			finish_servers.append(k)

	p = PROCESS_INFO.get(pair)
	if p:
		print("process:%d terminate:%s"%(p.pid, pair))
		p.terminate()
	print("server:%s terminate"%(finish_servers))
	msg[u"rtn"] = 1
	_send_msg(sock, msg)


def on_create_client(sock, msg):
	key = match_key(msg["proj"], msg["server_id"])
	pair = SERVER_INFO.get(key)
	msg[u"rtn"] = 0 if not pair else (u"tcp://%s:%d" % (IP, pair[0]))
	_send_msg(sock, msg)


def get_connected_server_node(sock, msg):
	"""请求当前broker上所有与项目相关的连接，projectID为自己的项目号, 为空显示全部"""
	projectID = msg["projectID"]
	ret = {}
	for serv in SERVER_INFO:
		if serv[0] == projectID:
			ret[serv[1]] = SERVER_INFO[serv]
	msg[u"rtn"] = ret
	_send_msg(sock, msg)


CMD_DISPATCH = {
	CMD_CREATE_SERVER: on_create_server,
	CMD_SERVER_QUIT: on_server_quit,
	CMD_CREATE_CLIENT: on_create_client,
	CMD_GET_SERVER_NODE: get_connected_server_node,
}

def main():
	ctx = zmq.Context()
	cmd_dealer = ctx.socket(zmq.REP)
	try:
		cmd_dealer.bind("tcp://%s:%d"%(BIND_IP, PORT))
	except zmq.ZMQError as e:
		en = e.errno
		if en == zmq.EADDRINUSE:
			print("port:%d is in use"%PORT)
		else:
			print("bind error:%d"%en)
		exit(-1)

	while True:
		try:
			msg = cmd_dealer.recv()
		except KeyboardInterrupt:
			print("Keyboard Ctrl+C")
			break

		try:
			msg = _unpack(msg)
		except:
			print("message error:%s"%(msg))
			continue

		deal = CMD_DISPATCH.get(msg["cmd"])
		if not deal:
			print("wrong cmd:%s got"%(str(msg)))
		else:
			deal(cmd_dealer, msg)

	for _, sub_proc in PROCESS_INFO.items():
		sub_proc.join()
		sub_proc.terminate()

	print("Broker exit peacefully.")


if __name__ == "__main__":
	if len(sys.argv) > 1:
		LOG_FILE = sys.argv[1]  # 日志目录
	main()



# coding=utf-8
__author__ = 'lnx3032'


import re

from axmlparserpy.apk import APK


def install(adb_client, localpath, force_reinstall=False):
    apk_info = APK(localpath)
    package_name = apk_info.package

    def _get_installed_apk_version(package):
        package_info = adb_client.shell(['dumpsys', 'package', package])
        matcher = re.search(r'versionCode=(\d+)', package_info)
        if matcher:
            return int(matcher.group(1))
        return None

    try:
        apk_version = int(apk_info.androidversion_code)
    except (RuntimeError, ValueError):
        apk_version = 0
    installed_version = _get_installed_apk_version(package_name)
    print('installed version is {}, installer version is {}. force_reinstall={}'.format(installed_version, apk_version, force_reinstall))
    if installed_version is None or apk_version > installed_version or force_reinstall:
        if installed_version is not None:
            force_reinstall = True
        if hasattr(adb_client, 'install_app'):
            adb_client.install_app(localpath, force_reinstall)
        else:
            adb_client.install(localpath, force_reinstall)
        return True
    return False


def uninstall(adb_client, package):
    if hasattr(adb_client, 'uninstall_app'):
        adb_client.uninstall_app(package)
    else:
        adb_client.uninstall(package)



# -*- coding: utf-8 -*-
# @Author: gzliuxin
# @Email:  gzliuxin@corp.netease.com
# @Date:   2017-07-14 19:47:51
from poco import Poco
from poco.agent import PocoAgent
from poco.freezeui.hierarchy import FreezedUIHierarchy, FreezedUIDumper
from poco.sdk.Attributor import Attributor
from poco.sdk.interfaces.screen import ScreenInterface
from poco.sdk.exceptions import UnableToSetAttributeException
from poco.utils.airtest import AirtestInput, AirtestScreen
from poco.utils.simplerpc.rpcclient import RpcClient
from poco.utils.simplerpc.transport.tcp.main import TcpClient
from poco.utils.simplerpc.utils import sync_wrapper

__all__ = ['UnityPoco']
DEFAULT_ADDR = ("localhost", 5001)


class UnityScreen(ScreenInterface):
    def __init__(self, client):
        super(UnityScreen, self).__init__()
        self.client = client

    @sync_wrapper
    def getScreen(self, width):
        return self.client.call("Screenshot", width)

    @sync_wrapper
    def getPortSize(self):
        return self.client.call("GetScreenSize")


class UnityPocoAgent(PocoAgent):
    def __init__(self, addr=DEFAULT_ADDR, unity_editor=False):
        if not unity_editor:
            # init airtest env
            try:
                # new version
                from airtest.core.api import connect_device, device as current_device
                if not current_device():
                    connect_device("Android:///")
            except ImportError:
                # old version
                from airtest.cli.runner import device as current_device
                from airtest.core.main import set_serialno
                if not current_device():
                    set_serialno()
            # unity games poco sdk listens on Android localhost:5001
            current_device().adb.forward("tcp:%s" % addr[1], "tcp:5001", False)

        self.conn = TcpClient(addr)
        self.c = RpcClient(self.conn)
        self.c.DEBUG = False
        self.c.run(backend=True)
        self.c.wait_connected()

        hierarchy = FreezedUIHierarchy(Dumper(self.c), UnityAttributor(self.c))
        if unity_editor:
            screen = UnityScreen(self.c)
        else:
            screen = AirtestScreen()
        input = AirtestInput()
        super(UnityPocoAgent, self).__init__(hierarchy, input, screen, None)

    @sync_wrapper
    def get_debug_profiling_data(self):
        return self.c.call("GetDebugProfilingData")


class UnityAttributor(Attributor):
    def __init__(self, client):
        super(UnityAttributor, self).__init__()
        self.client = client

    def setAttr(self, node, attrName, attrVal):
        if attrName == 'text':
            if type(node) in (list, tuple):
                node = node[0]
            instance_id = node.getAttr('_instanceId')
            if instance_id:
                success = self.client.call('SetText', instance_id, attrVal)
                if success:
                    return
        raise UnableToSetAttributeException(attrName, node)


class Dumper(FreezedUIDumper):
    def __init__(self, rpcclient):
        super(Dumper, self).__init__()
        self.rpcclient = rpcclient

    @sync_wrapper
    def dumpHierarchy(self):
        return self.rpcclient.call("Dump")


class UnityPoco(Poco):
    def __init__(self, addr=DEFAULT_ADDR, unity_editor=False, **options):
        agent = UnityPocoAgent(addr, unity_editor)
        if 'action_interval' not in options:
            options['action_interval'] = 0.1
        super(UnityPoco, self).__init__(agent, **options)

    def on_pre_action(self, action, proxy, args):
        try:
            from airtest.core.api import snapshot
        except ImportError:
            # 兼容旧airtest
            from airtest.core.main import snapshot
            snapshot(msg=unicode(proxy))



# encoding=utf-8
import sys

sys.path.append("../../..")
from simplerpc.simplerpc import dispatcher, AsyncResponse
from simplerpc.ssrpc.plugin import PluginRepo, Plugin, SSRpcServer, AgentManager
from simplerpc.transport.tcp import TcpServer
from pprint import pprint


@dispatcher.add_method
def foobar(**kwargs):
    return kwargs["foo"] + kwargs["bar"]


class AAAPlugin(Plugin):
    UUID = "AAAAAAA"

    def _on_rpc_ready(self, agent):
        # print agent.get_plugin(self.UUID).add(3, 5).wait()
        agent.get_plugin(self.UUID).add(3, 5).on_result(pprint)

    def minus(self, a, b):
        return a - b


class BBBPlugin(Plugin):
    UUID = "BBBBBBB"

    def echo(self, *args):
        return args


def test_ssrpc():
    PluginRepo.register(AAAPlugin())
    PluginRepo.register(BBBPlugin())
    AgentManager.ROLE = "SERVER"
    s = SSRpcServer(TcpServer())
    s.run()
    # s.console_run({"s": s})

if __name__ == '__main__':
    test_ssrpc()



# -*- coding: utf-8 -*-
"""----------------------------------------------------------------------------
Author:
	fengfan
	fengfan@corp.netease.com
Date:
	2017/1/10
Description:
	SSRPC module, Base RPC Module used in Sunshine Project
History:
	2017/1/10, create file.
----------------------------------------------------------------------------"""

from .SSRPC import init, stop, setup_server, setup_client, ssrpc_method, tick
from .SSRPC import SSRPCSvrHandler, SSRPCCliHandler, ZMQHandler
from .SSRPC import _RPCProxy as RPCProxy
from .SSRPC import MAIN_THREAD_TICK, MAIN_THREAD_SOCKET, MAIN_THREAD_EVERYTHING, SSRPC_ERR, SERV_MODE_ALL_GET_MSG



# coding=utf-8


import json
from hunter_cli import Hunter, open_platform
from poco.drivers.netease.internal import NeteasePoco

if __name__ == '__main__':
    tokenid = open_platform.get_api_token('poco-test')
    # hunter = Hunter(tokenid, 'g62', devid='g62_at_408d5c117d0f')
    hunter = Hunter(tokenid, 'g62', devid='g62_at_408d5c117d0f')
    poco = NeteasePoco('g62', hunter)

    # root = poco.agent.hierarchy.dumper.getRoot()
    # h = poco.agent.hierarchy.dumper.dumpHierarchyImpl(root, False)
    # print json.dumps(h)

    print poco(visible=False).query


# coding=utf-8

from poco.sdk.exceptions import UnableToSetAttributeException

__author__ = 'lxn3032'
__all__ = ['AbstractNode']


class AbstractNode(object):
    """
    AbstractNode is a wrapper class that provides UI hierarchy and node info in game engine.

    This class uniformly specifies node-related methods such as retrieving the attributes or accessing the parent nodes
    or their children.
    """

    def getParent(self):
        """
        Return the parent node of this node. Return None if there is no parent or parent is not accessible or this is
        the root node. This method is invoked by ``Selector`` or ``Dumper`` when traversing UI hierarchy.

        Returns:
            :py:class:`AbstractNode or NoneType <poco.sdk.AbstractNode>`: parent node of this node or None.
        """

        return None

    def getChildren(self):
        """
        Return an iterator over all children nodes of this node. This method is invoked by ``Selector`` or ``Dumper``
        to retrieve the UI hierarchy.

        Returns:
            Iterable <:py:class:`AbstractNode <poco.sdk.AbstractNode>`>
        """

        raise NotImplementedError

    def getAttr(self, attrName):
        """
        Return the attributes of the node. The list below shows the most used basic attributes used during while writing
        test code. The implementation class should return the corresponding value as soon as it retrieves its value. If
        the value cannot be determined, the default value is obtained from super class invocation and returned.  See the
        example below for more detailed information. More attributes can be added in order to enhance the selection
        and displaying in ``Inspector``.

        The most used basic attributes are listed as follows:

        - ``name``: name of the node, use the unique and meaningful name for each node is recommended
        - ``type``: type of the name of the node, it can be either any string, e.g. "android.widget.Button" or just
          simple as "Button"
        - ``visible``: True or False whether the node is rendered on screen. In case the return value is False, all
          children nodes are ignored in Poco selector
        - ``pos``: position of the node in screen, return value should be 2-list coordinates (x, y) representing
          the percentage of the screen. e.g. if the node lies in the center of the screen, the attribute will be
          ``[0.5f, 0.5f]``. If the returned value for position is negative, it means the node lies out of the screen
        - ``size``: size of the node bounding box, similar to ``pos``, value is also a 2-list of the percentage of
          the screen size, e.g. the screen size is always ``[1.0f, 1.0f]``,
          if the node lies in left half side of the screen, its size will be ``[0.5f, 1.0f]``, the returned value of
          size should be always positive value
        - ``scale``: scale factor applied to the node itself, leave it ``[1.0f, 1.0f]`` by default
        - ``anchorPoint``: 2-list coordinates (x, y) of the anchor expressed in the percentage related to the bounding
          box of the node, leave it ``[0.5f, 0.5f]`` by default.
        - ``zOrders``: rendering order of this node, its value is a dictionary such as ``{"global": 0, "local": 0}``,
          global zOrder value is compared with all nodes in the hierarchy, local zOrder value is compared with its
          parent and siblings. The most top nodes have the largest values.

        Examples:
            The following sample code demonstrates some ideas about the implementation of this method::
                
                def getAttr(self, attrName):
                    # basic attributes
                    if attrName == 'name':
                        return self.node.get_name() or '<no name>'

                    elif attrName == 'pos':
                        return self.node.get_position()

                    # ...

                    # extra engine-specific attributes
                    elif attrName == 'rotation':
                        return self.node.get_rotation()

                    # call the super method by default
                    else:
                        return super().getAttr(attrName)

        Args:
            attrName (:obj:`str`): attribute name

        Returns:
            JsonSerializable attribute value or None if no such attribute.
        """

        attrs = {
            'name': '<Root>',
            'type': 'Root',
            'visible': True,
            'pos': [0.0, 0.0],
            'size': [0.0, 0.0],
            'scale': [1.0, 1.0],
            'anchorPoint': [0.5, 0.5],
            'zOrders': {'local': 0, 'global': 0},
        }

        return attrs.get(attrName)

    def setAttr(self, attrName, val):
        """
        Apply changes of the attribute value to this node. Not all attributes can be modified. The most common
        attribute to be modified is the `text`. It is not recommended to modify attributes such as position, name,
        their modifications can lead to unexpected and false-positive errors.

        Args:
            attrName (:obj:`str`): attribute name
            val: attribute value

        See Also:
            :py:meth:`setAttr <poco.sdk.interfaces.hierarchy.HierarchyInterface.setAttr>` in
             ``poco.sdk.interfaces.hierarchy``
        """

        raise UnableToSetAttributeException(attrName, None)

    def getAvailableAttributeNames(self):
        """
        Enumerate all available attribute names of this node. This method in base class returns the basic
        attribute name by default. It is possible to add other customized or engine-specific attributes.
        See the example below.

        .. note:: It is recommended to always call the super method and return should contain the part from super method.

        Examples:
            This code demonstrates how to implement this method::

                def getAvailableAttributeNames(self):
                    return super().getAvailableAttributeNames() + (
                        # add other engine-specific attribute names here if need.
                        'rotation',
                    )

        Returns:
            Iterable<:obj:`str`>
        """

        return (
            "name",
            "type",
            "visible",
            "pos",
            "size",
            "scale",
            "anchorPoint",
            "zOrders",
        )

    def enumerateAttrs(self):
        """
        Enumerate all available attributes and yield the 2-tuple (name, value).

        Yields:
            Iterable<:obj:`str`, :obj:`ValueType`>
        """

        for attrName in self.getAvailableAttributeNames():
            yield attrName, self.getAttr(attrName)



# coding=utf-8
from ..interfaces import IConnection, IServer, IClient
from .asynctcp import Host, Client, init_loop
from .protocol import SimpleProtocolFilter


DEFAULT_ADDR = ("0.0.0.0", 5001)


class TcpConn(IConnection):
    """docstring for TcpConn"""
    def __init__(self, client):
        super(TcpConn, self).__init__()
        self.client = client
        self.prot = SimpleProtocolFilter()

    def send(self, msg):
        msg_bytes = self.prot.pack(msg)
        self.client.say(msg_bytes)

    def recv(self):
        msg_bytes = self.client.read_message()
        return self.prot.input(msg_bytes)


class TcpServer(IServer):
    def __init__(self, addr=DEFAULT_ADDR):
        super(TcpServer, self).__init__()
        self.s = Host(addr)
        # extend server's methods, not using multi-inheritance to prevent method name conflict
        self._s_handle_accept, self._s_close_client = self.s.handle_accept, self.s.close_client
        self.s.handle_accept, self.s.close_client = self._handle_accept, self._close_client
        self._connections = {}

    def start(self):
        init_loop()

    @property
    def connections(self):
        return self._connections

    def _handle_accept(self):
        client = self._s_handle_accept()
        conn = TcpConn(client)
        self._connections[client.cid] = conn
        self.on_client_connect(conn)

    def _close_client(self, client_id):
        client = self._s_close_client(client_id)
        conn = self._connections.pop(client.cid)
        self.on_client_close(conn)


class TcpClient(IClient):
    """docstring for TcpClient"""
    def __init__(self, addr=DEFAULT_ADDR):
        super(TcpClient, self).__init__()
        self.prot = SimpleProtocolFilter()
        self.c = Client(addr)
        # extends client's methods
        self._c_handle_connect, self._c_handle_close = self.c.handle_connect, self.c.handle_close
        self.c.handle_connect, self.c.handle_close = self._handle_connect, self._handle_close

    def connect(self):
        init_loop()

    def send(self, msg):
        msg_bytes = self.prot.pack(msg)
        self.c.say(msg_bytes)

    def recv(self):
        msg_bytes = self.c.read_message()
        return self.prot.input(msg_bytes)

    def _handle_connect(self):
        self._c_handle_connect()
        self.on_connect()

    def _handle_close(self):
        self._c_handle_close()
        self.on_close()



# -*- coding: utf-8 -*-
# @Author: gzliuxin
# @Email:  gzliuxin@corp.netease.com
# @Date:   2017-07-13 18:01:23
from poco.freezeui.hierarchy import FreezedUIHierarchy, FreezedUIDumper
from poco.sdk.interfaces.input import InputInterface
from poco.sdk.interfaces.screen import ScreenInterface
from poco.utils.simplerpc.utils import sync_wrapper


class MhScreen(ScreenInterface):
    def __init__(self, client):
        super(MhScreen, self).__init__()
        self.c = client

    @sync_wrapper
    def getPortSize(self):
        return self.c.call("get_size")

    @sync_wrapper
    def getScreen(self, width):
        # FIXME: get screen 是 get_size ？
        return self.c.call("get_size"), 'png'


class MhInput(InputInterface):
    def __init__(self, client):
        super(MhInput, self).__init__()
        self.c = client

    @sync_wrapper
    def click(self, x, y, op="left"):
        # FIXME: 这个函数签名不对
        print((x, y), op)
        return self.c.call("click", (x, y), op)

    def getTouchDownDuration(self):
        return 0.01


class MhDumper(FreezedUIDumper):
    def __init__(self, client):
        super(MhDumper, self).__init__()
        self.c = client

    @sync_wrapper
    def dumpHierarchy(self):
        return self.c.call("dump")


class MhHierarchy(FreezedUIHierarchy):
    def __init__(self, client):
        super(MhHierarchy, self).__init__(MhDumper(client))
        self.c = client

    def setAttr(self, node, name, val):
        if isinstance(node, (list, tuple)):
            node = node[0]
        node_id = node[0]["desc"]
        self._setattr(node_id, name, val)

    @sync_wrapper
    def _setattr(self, node_id, name, val):
        return self.c.call("setattr", node_id, name, val)


if __name__ == '__main__':
    from pprint import pprint
    r = MhHierarchy(addr=("10.254.245.124", 5001))
    # size = r.getPortSize()
    # r.click((0.5, 0.5))
    # dump = r.dump()
    # pprint(dump)
    # root = (dict_2_node(dump))
    # pprint(root)
    # pprint(root.getParent())
    # pprint(root.getChildren())
    nodes = r.select(["and", [["attr=", ["text", u"长安城"]]]])
    # nodes = r.select(["and",[["attr=",["type", "Text"]]]])
    pprint(nodes[0].node)



# -*- coding: utf-8 -*-
# @Author: gzliuxin
# @Email:  gzliuxin@corp.netease.com
# @Date:   2017-07-13 20:29:56
from poco import Poco
from poco.agent import PocoAgent
from poco.drivers.netease.mh.mh_rpc import MhHierarchy, MhScreen, MhInput
from poco.utils.simplerpc.rpcclient import RpcClient
from poco.utils.simplerpc.transport.tcp.main import TcpClient


class MhPocoAgent(PocoAgent):
    def __init__(self, addr=("localhost", 5001)):
        conn = TcpClient(addr)
        self.c = RpcClient(conn)
        self.c.DEBUG = False
        self.c.run(backend=True)

        hierarchy = MhHierarchy(self.c)
        screen = MhScreen(self.c)
        input = MhInput(self.c)
        super(MhPocoAgent, self).__init__(hierarchy, input, screen, None)


class MhPoco(Poco):
    """docstring for MhPoco"""
    def __init__(self, addr=("localhost", 5001), **options):
        agent = MhPocoAgent(addr)
        if 'action_interval' not in options:
            options['action_interval'] = 0.2
        super(MhPoco, self).__init__(agent, **options)



# coding=utf-8

try:
    from airtest.core.api import device as current_device
    from airtest.core.api import touch, swipe
    from airtest.core.helper import device_platform
except ImportError:
    # 兼容旧版本
    from airtest.cli.runner import device as current_device
    from airtest.core.main import touch, swipe
    from airtest.core.main import get_platform as device_platform
from poco.sdk.interfaces.input import InputInterface


class AirtestInput(InputInterface):
    def __init__(self, surface=None):
        super(AirtestInput, self).__init__()
        self.surface = surface  # {.getPortSize}
        self.default_touch_down_duration = 0.01

    def _get_touch_resolution(self):
        """
        get real time resolution on device if full screen
         or window size if running in window mode
        """
        if device_platform() == 'Windows':
            if self.surface is None:
                raise RuntimeError('Please initialize AirtestInput with surface object, '
                                   'when running test suites on windows as target device.')
            return self.surface.getPortSize()
        else:
            display_info = current_device().display_info
            w, h = display_info["width"], display_info["height"]
            if display_info["orientation"] in (1, 3):
                return h, w
            else:
                return w, h

    def setTouchDownDuration(self, duration):
        self.default_touch_down_duration = duration

    def getTouchDownDuration(self):
        return self.default_touch_down_duration

    def click(self, x, y):
        pw, ph = self._get_touch_resolution()
        pos = [x * pw, y * ph]
        touch(pos, duration=self.default_touch_down_duration)

    def swipe(self, x1, y1, x2, y2, duration=2.0):
        if duration <= 0:
            raise ValueError("Operation duration cannot be less equal 0. Please provide a positive number.")
        direction = x2 - x1, y2 - y1
        pw, ph = self._get_touch_resolution()
        p1 = [x1 * pw, y1 * ph]
        steps = int(duration * 40) + 1
        swipe(p1, vector=direction, duration=duration, steps=steps)

    def longClick(self, x, y, duration=2.0):
        if duration <= 0:
            raise ValueError("Operation duration cannot be less equal 0. Please provide a positive number.")
        pw, ph = self._get_touch_resolution()
        pos = [x * pw, y * ph]
        touch(pos, duration=duration)



# -*- coding: utf-8 -*-
# @Author: gzliuxin
# @Email:  gzliuxin@corp.netease.com
# @Date:   2017-07-14 19:47:51
import json

from poco import Poco
from poco.agent import PocoAgent
from poco.utils.simplerpc.utils import sync_wrapper
from poco.freezeui.hierarchy import FreezedUIHierarchy, FreezedUIDumper
from poco.utils.airtest import AirtestInput, AirtestScreen
from poco.utils.simplerpc.rpcclient import RpcClient
from poco.utils.simplerpc.transport.ws import WebSocketClient

__all__ = ['CocosJsPoco']
DEFAULT_ADDR = "ws://localhost:5003"


class CocosJsPocoAgent(PocoAgent):
    def __init__(self, addr=DEFAULT_ADDR):
        # init airtest env
        try:
            # new version
            from airtest.core.api import connect_device, device as current_device
            if not current_device():
                connect_device("Android:///")
        except ImportError:
            # old version
            from airtest.cli.runner import device as current_device
            from airtest.core.main import set_serialno
            if not current_device():
                set_serialno()
        # cocos games poco sdk listens on Android localhost:5003
        localport = addr.split(":")[-1]
        current_device().adb.forward("tcp:%s" % localport, "tcp:5003", False)

        self.conn = WebSocketClient(addr)
        self.c = RpcClient(self.conn)
        self.c.DEBUG = False
        self.c.run(backend=True)
        self.c.wait_connected()

        hierarchy = FreezedUIHierarchy(Dumper(self.c))
        screen = AirtestScreen()
        input = AirtestInput()
        super(CocosJsPocoAgent, self).__init__(hierarchy, input, screen, None)


class Dumper(FreezedUIDumper):
    def __init__(self, rpcclient):
        super(Dumper, self).__init__()
        self.rpcclient = rpcclient

    @sync_wrapper
    def dumpHierarchy(self):
        return self.rpcclient.call("dump")


class CocosJsPoco(Poco):
    """docstring for CocosJsPoco"""

    def __init__(self, addr=DEFAULT_ADDR, **options):
        agent = CocosJsPocoAgent(addr)
        if 'action_interval' not in options:
            options['action_interval'] = 0.1
        super(CocosJsPoco, self).__init__(agent, **options)

    def on_pre_action(self, action, proxy, args):
        # airteset log用
        try:
            from airtest.core.api import snapshot
        except ImportError:
            # 兼容旧airtest
            from airtest.core.main import snapshot
        snapshot(msg=unicode(proxy))


# test code
def dump():
    from websocket import create_connection
    ws = create_connection(DEFAULT_ADDR, timeout=2)
    ws.send('{"jsonrpc": "2.0", "params": {}, "method": "dump", "id": 0}')
    # print("Sent")
    # print("Receiving...")
    result = ws.recv()
    # print("Received '%s'" % result)
    ws.close()
    data = json.loads(result)
    return data["result"]



""" Test utility functionality."""
import datetime
import decimal
import json
import sys
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from mock import patch

from ..utils import JSONSerializable, DatetimeDecimalEncoder, is_invalid_params


class TestJSONSerializable(unittest.TestCase):

    """ Test JSONSerializable functionality."""

    def setUp(self):
        class A(JSONSerializable):
            @property
            def json(self):
                pass

        self._class = A

    def test_abstract_class(self):
        with self.assertRaises(TypeError):
            JSONSerializable()

        self._class()

    def test_definse_serialize_deserialize(self):
        """ Test classmethods of inherited class."""
        self.assertEqual(self._class.serialize({}), "{}")
        self.assertEqual(self._class.deserialize("{}"), {})

    def test_from_json(self):
        self.assertTrue(isinstance(self._class.from_json('{}'), self._class))

    def test_from_json_incorrect(self):
        with self.assertRaises(ValueError):
            self._class.from_json('[]')


class TestDatetimeDecimalEncoder(unittest.TestCase):

    """ Test DatetimeDecimalEncoder functionality."""

    def test_date_encoder(self):
        obj = datetime.date.today()

        with self.assertRaises(TypeError):
            json.dumps(obj)

        self.assertEqual(
            json.dumps(obj, cls=DatetimeDecimalEncoder),
            '"{0}"'.format(obj.isoformat()),
        )

    def test_datetime_encoder(self):
        obj = datetime.datetime.now()

        with self.assertRaises(TypeError):
            json.dumps(obj)

        self.assertEqual(
            json.dumps(obj, cls=DatetimeDecimalEncoder),
            '"{0}"'.format(obj.isoformat()),
        )

    def test_decimal_encoder(self):
        obj = decimal.Decimal('0.1')

        with self.assertRaises(TypeError):
            json.dumps(obj)

        result = json.dumps(obj, cls=DatetimeDecimalEncoder)
        self.assertTrue(isinstance(result, str))
        self.assertEqual(float(result), float(0.1))

    def test_default(self):
        encoder = DatetimeDecimalEncoder()
        with patch.object(json.JSONEncoder, 'default') as json_default:
            encoder.default("")

        self.assertEqual(json_default.call_count, 1)


class TestUtils(unittest.TestCase):

    """ Test utils functions."""

    def test_is_invalid_params_builtin(self):
        self.assertTrue(is_invalid_params(sum, 0, 0))
        # NOTE: Function generates TypeError already
        # self.assertFalse(is_invalid_params(sum, [0, 0]))

    def test_is_invalid_params_args(self):
        self.assertTrue(is_invalid_params(lambda a, b: None, 0))
        self.assertTrue(is_invalid_params(lambda a, b: None, 0, 1, 2))

    def test_is_invalid_params_kwargs(self):
        self.assertTrue(is_invalid_params(lambda x: None, **{}))
        self.assertTrue(is_invalid_params(lambda x: None, **{"x": 0, "y": 1}))

    def test_invalid_params_correct(self):
        # self.assertFalse(is_invalid_params(lambda: None))
        self.assertFalse(is_invalid_params(lambda a: None, 0))
        self.assertFalse(is_invalid_params(lambda a, b=0: None, 0))



# coding=utf-8

import time
from poco.exceptions import PocoTargetTimeout
from poco.drivers.unity3d.test.tutorial.case import TutorialCase


class InvalidOperationExceptionTutorial(TutorialCase):
    def runTest(self):
        # UI is very slow
        self.poco('btn_start').click()
        star = self.poco('star')
        try:
            star.wait_for_appearance(timeout=3)  # wait until appearance within 3s
        except PocoTargetTimeout:
            print('oops!')
            time.sleep(1)


if __name__ == '__main__':
    from airtest.core.api import connect_device
    connect_device('Android:///')
    import pocounit
    pocounit.main()



# coding=utf-8

from poco.sdk.interfaces.command import CommandInterface

__author__ = 'lxn3032'


class HunterCommand(CommandInterface):
    def __init__(self, hunter):
        self.hunter = hunter

    def command(self, cmd, type=None):
        """
        通过hunter调用gm指令，可调用hunter指令库中定义的所有指令，也可以调用text类型的gm指令
        gm指令相关功能请参考safaia GM指令扩展模块

        :param script: 指令
        :param lang: 语言，默认text
        :param sleep_interval: 调用指令后的等待间隔时间
        :return: None
        """

        type = type or 'text'
        self.hunter.script(cmd, lang=type)



from __future__ import print_function

import asyncore
import collections
import socket


LOOP_THREAD = None
MAX_MESSAGE_LENGTH = 4096


class BaseClient(asyncore.dispatcher):

    """base class for both local/remote client socket."""

    def __init__(self):
        asyncore.dispatcher.__init__(self)
        self.outbox = collections.deque()
        self.inbox = ""
        self.closed = False

    def say(self, message):
        self.outbox.append(message)

    def writable(self):
        if self.connecting:  # is it a bug in asynccore? handle_connect_event triggered only when writable is True
            return True
        return bool(self.outbox)

    def handle_write(self):
        if not self.outbox:
            return
        message = self.outbox.popleft()
        self.send(message)

    def handle_read(self):
        message = self.recv(MAX_MESSAGE_LENGTH)
        if message == b"":
            return
        self.inbox += message

    def handle_close(self):
        asyncore.dispatcher.handle_close(self)
        print("Closed", self)
        self.closed = True

    def read_message(self, length=None):
        if length is None:
            message, self.inbox = self.inbox, ""
        else:
            message, self.inbox = self.inbox[:length], self.inbox[length:]
        return message


class RemoteClient(BaseClient):

    """Wraps a remote client socket."""

    def __init__(self, host, socket, address, cid=None):
        BaseClient.__init__(self)
        self.host = host
        self.cid = cid
        self.set_socket(socket)

    def handle_close(self):
        BaseClient.handle_close(self)
        self.host.close_client(self.cid)


class Host(asyncore.dispatcher):

    CLIENT_ID_COUNTER = 1
    CLIENT_MAX_COUNT = 10

    def __init__(self, address=('0.0.0.0', 5001)):
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.bind(address)
        self.listen(self.CLIENT_MAX_COUNT)
        print('server listening at', address)
        self.remote_clients = {}
        self.msg_queue = []

    def handle_accept(self):
        socket, addr = self.accept()  # For the remote client.
        print('Accepted client at %s', addr)
        client_id = self.__class__.CLIENT_ID_COUNTER
        self.__class__.CLIENT_ID_COUNTER += 1
        client = RemoteClient(self, socket, addr, client_id)
        self.remote_clients[client_id] = client
        print('connected clients', self.remote_clients)
        return client

    def swap_msg_queue(self):
        for client in self.remote_clients.values():
            message = client.read_message()
            self.msg_queue.append(message)

        msg_queue = self.msg_queue
        self.msg_queue = []
        return msg_queue

    def broadcast(self, message):
        print('Broadcasting message: %s' % len(message))
        for remote_client in self.remote_clients.values():
            remote_client.say(message)

    def say(self, client_id, message):
        self.remote_clients[client_id].say(message)

    def close_client(self, client_id):
        print('Closing client:', client_id, self.remote_clients)
        client = self.remote_clients.pop(client_id)
        return client


class Client(BaseClient):

    def __init__(self, host_address):
        BaseClient.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        print('Connecting to host at:', host_address)
        self.connect(host_address)

    def handle_connect(self):
        print(self, " is connected")


def start_thread(target, *args, **kwargs):
    import threading
    t = threading.Thread(target=target, args=args, kwargs=kwargs)
    t.daemon = True
    t.start()
    return t


def init_loop():
    global LOOP_THREAD
    if LOOP_THREAD and LOOP_THREAD.is_alive():
        print("LOOP_THREAD ALREADY STARTED: %s" % LOOP_THREAD)
    else:
        LOOP_THREAD = start_thread(asyncore.loop, timeout=0.001)


if __name__ == '__main__':
    h = Host()
    asyncore.loop()



# -*- coding: utf-8 -*-
# @Author: gzliuxin
# @Email:  gzliuxin@corp.netease.com
# @Date:   2017-07-14 19:48:10
from poco.drivers.cocosjs import CocosJsPoco, CocosJsPocoAgent
from pprint import pprint
import json
import time


def quick_game(p):
    p("btnQuickStart").click('center')

    for i in range(4):
        p("btnChip%d" % i).click('center')

    p("btnBet").click('center')
    p("btnBack").click('center')


if __name__ == '__main__':
    rpc = CocosJsPocoAgent()
    t0 = time.time()
    rpc.hierarchy.dump()
    t1 = time.time()
    print t1 - t0
    # print(dump())

    '''adb reverse tcp:5002 tcp:5002'''
    # addr = ('', 5002)
    # r = SocketIORpc(addr)
    # d = (r.dump())
    # pprint(d)

    # 默认的就是5001
    '''adb reverse tcp:5001 tcp:5001'''
    # r = SocketIORpc(addr)
    # d = (r.dump())
    # with open("test.log", "w") as f:
    #     f.write(json.dumps(d, indent=4))
    p = CocosJsPoco()
    # ret = p._rpc_client.dump()
    # pprint(ret)
    for i in range(300):
        print(i * 100000000000)
        quick_game(p)



# coding=utf-8
from __future__ import unicode_literals

import six


__author__ = 'lxn3032'


class PocoException(Exception):
    """
    Base class for errors and exceptions of Poco. It is Python3 compatible.
    """

    def __str__(self):
        if six.PY2:
            if isinstance(self.message, unicode):
                return self.message.encode("utf-8")
            else:
                return self.message
        else:
            if isinstance(self.message, bytes):
                return self.message.decode('utf-8')
            else:
                return self.message

    __repr__ = __str__


class InvalidOperationException(PocoException):
    """
    Raised when the operation performing on target device is foreseen, e.g. instruction to click outside the screen is
    definitely meaningless, then the ``InvalidOperationException`` is raised.
    """

    pass


class PocoTargetTimeout(PocoException):
    """
    Raised when the timeout expired while waiting for some condition to be fulfilled, e.g. waiting for the specific
    UI element but it has not appeared on the screen.
    """

    def __init__(self, action, poco_obj_proxy):
        super(PocoTargetTimeout, self).__init__()
        self.message = 'Waiting timeout for {} of "{}"'.format(action, poco_obj_proxy)


class PocoNoSuchNodeException(PocoException):
    """
    Raised when the UI element specified by query expression cannot be found.
    """

    def __init__(self, objproxy):
        super(PocoNoSuchNodeException, self).__init__()
        self.message = 'Cannot find any visible node by query {}'.format(objproxy)


class PocoTargetRemovedException(PocoException):
    """
    Raised when the hierarchy structure changed over the selection or when accessing the UI element that is already
    recycled.

    In most cases, there is no need to handle this exception manually. If this exception occurs, it usually means it
    is a bug in your code rather than application itself. Check your code first. The most of misuses comes from as
    follows.
    
    Examples:
        ::

            button1 = poco('button1')
            time.sleep(10)   # waiting for long enough before the UI hierarchy changing
            button1.click()  # PocoTargetRemovedException will raise at this line. Because the 'button1' is not on the screen.
    """

    def __init__(self, action, objproxy):
        super(PocoTargetRemovedException, self).__init__()
        self.message = 'Remote ui object "{}" has been removed from hierarchy during {}.'.format(objproxy, action)



# encoding=utf-8
import inspect
from ..simplerpc import dispatcher, RpcAgent
from ..rpcclient import RpcClient
from ..rpcserver import RpcServer
from functools import partial


class PluginRepo(object):

    REPO = {}

    @classmethod
    def register(cls, plugin):
        cls.REPO[plugin.UUID] = plugin
        methods = inspect.getmembers(plugin, predicate=inspect.ismethod)
        for name, method in methods:
            if not name.startswith("_"):
                # add all plugin methods
                plugin_funcname = "_%s_%s" % (plugin.UUID, name)
                print("register rpc method", plugin_funcname, method)
                dispatcher.add_method(method, plugin_funcname)

    @classmethod
    def plugins_ready(cls, agent):
        def cb():
            AgentManager.register(agent)
            for pid, plugin in cls.REPO.items():
                plugin._on_rpc_ready(agent)
        agent.get_role_from_remote(cb)

    @classmethod
    def plugins_close(cls, agent):
        if not agent.registered:
            return
        AgentManager.unregister(agent)
        for pid, plugin in PluginRepo.REPO.items():
            plugin._on_rpc_close(agent)


class Plugin(object):

    UUID = None

    def _on_rpc_ready(self, agent):
        # print("on_rpc_ready", self, agent)
        pass

    def _on_rpc_close(self, agent):
        # print("on_rpc_close", self, agent)
        pass


class SSRpcClient(RpcClient):
    def on_connect(self):
        super(SSRpcClient, self).on_connect()
        agent = RemoteAgent(self, self.call)
        PluginRepo.plugins_ready(agent)

    def on_close(self):
        super(SSRpcClient, self).on_close()
        agent = AgentManager.get_agent_by_conn(self)
        if agent:
            PluginRepo.plugins_close(agent)


class SSRpcServer(RpcServer):
    def on_client_connect(self, conn):
        super(SSRpcServer, self).on_client_connect(conn)
        agent = RemoteAgent(conn, partial(self.call, conn))
        # 这里不要rpc.wait，会卡死，因为这个函数返回了连接才建立
        PluginRepo.plugins_ready(agent)

    def on_client_close(self, conn):
        super(SSRpcServer, self).on_client_close(conn)
        agent = AgentManager.get_agent_by_conn(conn)
        if agent:
            PluginRepo.plugins_close(agent)


class RemoteAgent(RpcAgent):
    def __init__(self, conn, rpccall):
        super(RemoteAgent, self).__init__()
        self.conn = conn
        self.rpccall = rpccall
        self.role = None
        self.registered = False

    def call(self, *args, **kwargs):
        return self.rpccall(*args, **kwargs)

    def get_role_from_remote(self, cb):
        def func(role):
            self.role = role
            cb()
        self.call("get_role").on_result(func)

    def get_plugin(self, uuid):
        return RemotePlugin(uuid, agent=self)


class AgentManager(object):

    REPO = []
    MAIN = None
    ROLE = None  # ["SERVER", "CLIENT", ...]

    @classmethod
    def register(cls, agent):
        cls.REPO.append(agent)
        if not cls.MAIN:
            cls.MAIN = agent
        agent.registered = True

    @classmethod
    def unregister(cls, agent):
        cls.REPO.remove(agent)
        if cls.MAIN == agent and cls.REPO:
            print("Main Agent Changed")
            cls.MAIN = cls.REPO[0]
        agent.registered = False

    @classmethod
    def get_agent_by_conn(cls, conn):
        for i in cls.REPO:
            if i.conn == conn:
                return i
        return None

    @classmethod
    def get_agents(cls, role=None):
        if role is None:
            return cls.REPO
        else:
            return [a for a in cls.REPO if a.role == role]

    @classmethod
    def set_main_agent(cls, agent):
        cls.MAIN = agent

    @classmethod
    def get_main_agent(cls):
        return cls.MAIN

    @staticmethod
    @dispatcher.add_method
    def get_role():
        """return role for remote"""
        return AgentManager.ROLE
    

class RemotePlugin(object):
    def __init__(self, uuid, agent=None, agent_filter=None):
        self._uuid = uuid
        self._agent = agent
        self._filter = agent_filter

    def _call_rpc(self, agent, funcname, *args, **kwargs):
        plugin_funcname = "_%s_%s" % (self._uuid, funcname)
        print(agent, plugin_funcname)
        return agent.call(plugin_funcname, *args, **kwargs)

    def _all(self, role=None):
        agent_filter = partial(AgentManager.get_agents, role)
        obj = RemotePlugin(self._uuid, agent_filter=agent_filter)
        return obj

    def _make_rpc_call(self, funcname, *args, **kwargs):
        if self._agent:
            return self._call_rpc(self._agent, funcname, *args, **kwargs)
        elif self._filter is None:
            return self._call_main_rpc(funcname, *args, **kwargs)
        else:
            agents = self._filter()
            cbs = []
            for agent in agents:
                cb = self._call_rpc(agent, funcname, *args, **kwargs)
                cbs.append(cb)
            # 返回cbs。。不太好，有更好的搞法吗？
            # cbs共用一个对象？各自callback？
            return cbs

    def _call_main_rpc(self, funcname, *args, **kwargs):
        agent = AgentManager.MAIN
        assert agent is not None, "Main Agent not set yet"
        return self._call_rpc(agent, funcname, *args, **kwargs)

    def __getattr__(self, funcname):
        def _call(*args, **kwargs):
            return self._make_rpc_call(funcname, *args, **kwargs)
        return _call



# coding=utf-8
__author__ = 'lxn3032'


import os
import requests
import time
import warnings

try:
    new_airtest_api = True
    import airtest.core.api as _____
except ImportError:
    new_airtest_api = False
    from airtest.core.android.utils.iputils import get_ip_address

from airtest.core.android.ime import YosemiteIme


from hrpc.client import RpcClient
from hrpc.transport.http import HttpTransport
from poco import Poco
from poco.agent import PocoAgent
from poco.sdk.Attributor import Attributor
from poco.utils.hrpc.hierarchy import RemotePocoHierarchy
from poco.drivers.android.utils.installation import install, uninstall

__all__ = ['AndroidUiautomationPoco', 'AndroidUiautomationHelper']
this_dir = os.path.dirname(os.path.realpath(__file__))
PocoServicePackage = 'com.netease.open.pocoservice'
PocoServicePackageTest = 'com.netease.open.pocoservice.test'


class AndroidRpcClient(RpcClient):
    def __init__(self, endpoint):
        self.endpoint = endpoint
        super(AndroidRpcClient, self).__init__(HttpTransport)

    def initialize_transport(self):
        return HttpTransport(self.endpoint, self)


class AttributorWrapper(Attributor):
    """
    部分手机上仍不支持Accessibility.ACTION_SET_TEXT，使用YosemiteIme还是兼容性最好的方案
    这个class会hook住set_text，然后改用ime的text方法
    """

    def __init__(self, remote, ime):
        self.remote = remote
        self.ime = ime

    def getAttr(self, node, attrName):
        return self.remote.getAttr(node, attrName)

    def setAttr(self, node, attrName, attrVal):
        if attrName == 'text':
            self.ime.text(attrVal)
        else:
            self.remote.setAttr(node, attrName, attrVal)


class AndroidPocoAgent(PocoAgent):
    def __init__(self, endpoint, ime):
        self.client = AndroidRpcClient(endpoint)
        remote_poco = self.client.remote('poco-uiautomation-framework')
        dumper = remote_poco.dumper
        selector = remote_poco.selector
        attributor = AttributorWrapper(remote_poco.attributor, ime)
        hierarchy = RemotePocoHierarchy(dumper, selector, attributor)
        super(AndroidPocoAgent, self).__init__(hierarchy, remote_poco.inputer, remote_poco.screen, None)


class AndroidUiautomationPoco(Poco):
    def __init__(self, device=None, using_proxy=True, **options):
        if not device:
            try:
                # new version
                from airtest.core.api import connect_device, device as current_device
                if not current_device():
                    connect_device("Android:///")
            except ImportError:
                # old version
                from airtest.cli.runner import device as current_device
                from airtest.core.main import set_serialno
                if not current_device():
                    set_serialno()
            self.android = current_device()
        else:
            self.android = device
        self.adb_client = self.android.adb
        if using_proxy:
            self.device_ip = self.adb_client.host or "127.0.0.1"
        else:
            if new_airtest_api:
                self.device_ip = self.adb_client.get_ip_address()
            else:
                self.device_ip = get_ip_address(self.adb_client)

        # save current top activity (@nullable)
        current_top_activity_package = self.android.get_top_activity_name()
        if current_top_activity_package is not None:
            current_top_activity_package = current_top_activity_package.split('/')[0]

        # install ime
        self.ime = YosemiteIme(self.adb_client)
        self.ime.start()

        # install
        self._instrument_proc = None
        self._install_service()

        # forward
        if using_proxy:
            p0, _ = self.adb_client.setup_forward("tcp:10080")
            p1, _ = self.adb_client.setup_forward("tcp:10081")
        else:
            p0 = 10080
            p1 = 10081

        # start
        if self._is_running('com.github.uiautomator'):
            warnings.warn('{} should not run together with "uiautomator". "uiautomator" will be killed.'
                          .format(self.__class__.__name__))
            self.adb_client.shell(['am', 'force-stop', 'com.github.uiautomator'])

        ready = self._start_instrument(p0)
        if not ready:
            # 启动失败则需要卸载再重启，instrument的奇怪之处
            uninstall(self.adb_client, PocoServicePackage)
            self._install_service()
            ready = self._start_instrument(p0)

            if current_top_activity_package is not None:
                current_top_activity2 = self.android.get_top_activity_name()
                if current_top_activity2 is None or current_top_activity_package not in current_top_activity2:
                    self.android.start_app(current_top_activity_package, activity=True)

            if not ready:
                raise RuntimeError("unable to launch AndroidUiautomationPoco")

        endpoint = "http://{}:{}".format(self.device_ip, p1)
        agent = AndroidPocoAgent(endpoint, self.ime)
        super(AndroidUiautomationPoco, self).__init__(agent, **options)

    def _install_service(self):
        updated = install(self.adb_client, os.path.join(this_dir, 'lib', 'pocoservice-debug.apk'))
        install(self.adb_client, os.path.join(this_dir, 'lib', 'pocoservice-debug-androidTest.apk'), updated)
        return updated

    def _is_running(self, package_name):
        processes = self.adb_client.shell(['ps']).splitlines()
        for ps in processes:
            ps = ps.strip()
            if ps.endswith(package_name):
                return True
        return False

    # def _keep_running_instrumentation(self):
    #     def loop():
    #         while True:
    #             proc = self.adb_client.shell([
    #                 'am', 'instrument', '-w', '-e', 'class',
    #                 '{}.InstrumentedTestAsLauncher#launch'.format(PocoServicePackage),
    #                 '{}.test/android.support.test.runner.AndroidJUnitRunner'.format(PocoServicePackage)],
    #                 not_wait=True)
    #             stdout, stderr = proc.communicate()
    #             print(stdout)
    #             print(stderr)
    #             time.sleep(1)
    #     t = threading.Thread(target=loop)
    #     t.daemon = True
    #     t.start()

    def _start_instrument(self, port_to_ping):
        if self._instrument_proc is not None:
            self._instrument_proc.kill()
            self._instrument_proc = None
        ready = False
        self.adb_client.shell(['am', 'force-stop', PocoServicePackage])

        # 启动instrument之前，先把主类activity启动起来，不然instrumentation可能失败
        self.adb_client.shell('am start -n {}/.TestActivity'.format(PocoServicePackage))

        instrumentation_cmd = [
                'am', 'instrument', '-w', '-e', 'debug', 'false', '-e', 'class',
                '{}.InstrumentedTestAsLauncher'.format(PocoServicePackage),
                '{}.test/android.support.test.runner.AndroidJUnitRunner'.format(PocoServicePackage)]
        if new_airtest_api:
            self._instrument_proc = self.adb_client.start_shell(instrumentation_cmd)
        else:
            self._instrument_proc = self.adb_client.shell(instrumentation_cmd, not_wait=True)
        time.sleep(2)
        for i in range(10):
            try:
                requests.get('http://{}:{}'.format(self.device_ip, port_to_ping), timeout=10)
                ready = True
                break
            except requests.exceptions.Timeout:
                break
            except requests.exceptions.ConnectionError:
                time.sleep(1)
                print("still waiting for uiautomation ready.")
                continue
        return ready

    def on_pre_action(self, action, proxy, args):
        # airteset log用
        try:
            from airtest.core.api import snapshot
        except ImportError:
            # 兼容旧airtest
            from airtest.core.main import snapshot
        snapshot(msg=unicode(proxy))


class AndroidUiautomationHelper(object):
    _nuis = {}

    @classmethod
    def get_instance(cls, device):
        if cls._nuis.get(device) is None:
            cls._nuis[device] = AndroidUiautomationPoco(device)
        return cls._nuis[device]



# coding=utf-8
from .SSRPC import setup_client, setup_server, init, tick, SERV_MODE_ALL_GET_MSG, ZMQHandler
from .SSRPC.SSRPC import _pack, _HEARTBEAT_DATA, _HEARTBEAT_SEC
from ..interfaces import IServer, IClient, IConnection
import time



class ServerHandler(ZMQHandler):

    def __init__(self, client_connect_cb=None, client_close_cb=None):
        super(ServerHandler, self).__init__()
        self.on_client_connect = client_connect_cb
        self.on_client_close = client_close_cb
        self.online_clients = {}

    def on_init_finish(self, rtn):
        print("on_init_finish", rtn)

    def _on_multi_data(self, msgs):
        self._zmq_cache_msgs.extend(msgs)
        while len(self._zmq_cache_msgs) >= 2:
            cid = self._zmq_cache_msgs.pop(0)
            data = self._zmq_cache_msgs.pop(0)
            # print(cid, data)
            if cid == b"myclients_cb":  # 特殊信息
                continue

            conn = self.online_clients.get(cid)
            if not conn:
                conn = SSZmqConn(cid, self)
                self.online_clients[cid] = conn
                self.on_client_connect(cid)
            conn.lastHeartBeat = time.time()
            if data != _HEARTBEAT_DATA:
                data = self._unpack_data(data)
                conn.inbox.append(data)

    def send(self, client_id, data):
        self._zmq_socket.send_multipart([client_id, self._pack_data(data)])

    def send_heartbeat(self):
        self._zmq_socket.send_multipart([_HEARTBEAT_DATA, _pack(self._my_id)])

        for cli in list(self.online_clients):
            if time.time() - self.online_clients[cli].lastHeartBeat > _HEARTBEAT_SEC * 2:
                self.on_client_close(cli)
                self.online_clients.pop(cli)


class ClientHandler(ZMQHandler):
    def __init__(self, connect_cb=None, close_cb=None):
        super(ClientHandler, self).__init__()
        self.inbox = []
        self.on_connect = connect_cb
        self.on_close = close_cb

    def on_init_finish(self, rtn):
        print("on_init_finish", self.on_connect, rtn)
        if callable(self.on_connect) and rtn is True:
            self.on_connect()

    def _on_raw_data(self, data):
        data = self._unpack_data(data)
        self.inbox.append(data)

    def send(self, data):
        if hasattr(self, "_zmq_socket") and self._zmq_socket is not None:
            self._zmq_socket.send(self._pack_data(data))

    def send_heartbeat(self):
        self._zmq_socket.send(_HEARTBEAT_DATA)

    def recv(self):
        msgs, self.inbox = self.inbox, []
        return msgs


class SSZmqConn(IConnection):
    def __init__(self, cid, sh):
        self.cid = cid
        self.sh = sh
        self.inbox = []
        self.lastHeartbeat = 0

    def send(self, msg):
        self.sh.send(self.cid, msg)

    def recv(self):
        msgs, self.inbox = self.inbox, []
        return msgs


class SSZmqServer(IServer):
    def __init__(self, addr=("g68", "ff_server")):
        super(SSZmqServer, self).__init__()
        self.addr = addr
        self.sh = ServerHandler(client_connect_cb=self.on_client_connect, client_close_cb=self.on_client_close)

    def start(self):
        init(self.addr[0], err_handler=init_err_handler)
        setup_server(self.addr[1], self.sh, SERV_MODE_ALL_GET_MSG)

    @property
    def connections(self):
        return self.sh.online_clients


class SSZmqClient(IClient):
    def __init__(self, addr=("g68", "ff_server")):
        super(SSZmqClient, self).__init__()
        self.addr = addr
        self.ch = ClientHandler(connect_cb=self.on_connect, close_cb=self.on_close)  # close_cb notimplemented yet

    def connect(self):
        init(self.addr[0], err_handler=init_err_handler)
        setup_client(self.addr[1], self.ch)

    def send(self, msg):
        self.ch.send(msg)

    def recv(self):
        return self.ch.recv()


def init_err_handler(code, msg):
    print("my_err_handler %d %s"%(code, msg))



# coding=utf-8

from poco.drivers.unity3d.test.tutorial.case import TutorialCase


class Scroll1Tutorial(TutorialCase):
    def runTest(self):
        # swipe the list view up
        self.poco('Scroll View').swipe([0, -0.1])
        self.poco('Scroll View').swipe('up')  # the same as above, also have down/left/right
        self.poco('Scroll View').swipe('down')

        # perform swipe without UI selected
        x, y = self.poco('Scroll View').get_position()
        end = [x, y - 0.1]
        dir = [0, -0.1]
        self.poco.swipe([x, y], end)  # drag from point A to point B
        self.poco.swipe([x, y], direction=dir)  # drag from point A toward given direction and length


if __name__ == '__main__':
    from airtest.core.api import connect_device
    connect_device('Android:///')
    import pocounit
    pocounit.main()



# -*- coding: utf-8 -*-
"""----------------------------------------------------------------------------
Author:
	fengfan
	fengfan@corp.netease.com
Date:
	2017/1/10
Description:
	Sunshine RPC Module
History:
	2017/1/10, create file.
----------------------------------------------------------------------------"""
import sys
import uuid

is_py2 = sys.version[0] == '2'
if is_py2:
	from Queue import Queue, Empty
else:
	from queue import Queue, Empty

import msgpack
import zmq
import threading
import time

CMD_CREATE_SERVER = 1
CMD_CREATE_CLIENT = 2
CMD_SERVER_QUIT = 3
CMD_GET_SERVER_NODE = 4
_pack = lambda dat: msgpack.packb(dat, use_bin_type=True)
_unpack = lambda dat: msgpack.unpackb(dat, encoding="utf-8")

_zmq_context = None
_zmq_poll = None
_zmq_cmd_sock = None

_cmd_thread = None
_socket_thread = None
_project = None
_err_handler = None


class SSRPC_ERR(object):
	CONNECT_FAILED = 1  # 连接失败

def _err_msg(code, msg):
	if _err_handler:
		_err_handler(code, msg)
	else:
		# import traceback
		# traceback.print_stack()
		print("SSRPC errno:%d msg:%s"%(code, msg))

def _hex_str(tstr):
	return ":".join(hex(x if isinstance(x, int) else ord(x))[2:] for x in tstr)

_cmd_queue = Queue()
def cmd_loop(cmd_q, cmd_sock):
	while True:
		cmd = cmd_q.get()
		if isinstance(cmd, int):  # 退出
			break
		cmd_sock.send(_pack(cmd[0]))

		msg = cmd_sock.recv()
		msg = _unpack(msg)
		if cmd[1]:
			cmd[1](msg)

	# break out here
	cmd_sock.close()

_HEARTBEAT_SEC = 5
_HEARTBEAT_DATA = b'_ss_inner_heartbeat'
hb_time = 0
_sock_list = {}

def socket_loop(poller):
	hb_time = time.time() + _HEARTBEAT_SEC
	while True:
		socks = dict(poller.poll(_HEARTBEAT_SEC/2*1000))

		for s in socks:
			handler = _sock_list.get(s)
			if not handler:
				print(s, "has no handler???")
				continue

			if handler.is_server:
				msgs = s.recv_multipart()  # 只有两段，[ router对client分配的id，数据 ]，目前不允许发多段数据
				handler._on_multi_data(msgs)
			else:
				msg = s.recv()
				handler._on_raw_data(msg)

		if hb_time < time.time():  # 发送心跳包
			hb_time = time.time() + _HEARTBEAT_SEC
			for handler in _sock_list.values():
				handler.send_heartbeat()


def socket_tick():
	poller = _zmq_poll
	socks = dict(poller.poll(_HEARTBEAT_SEC / 2))

	for s in socks:
		handler = _sock_list.get(s)
		if not handler:
			print(s, "has no handler???")
			continue

		if handler.is_server:
			msgs = s.recv_multipart()  # 只有两段，[ router对client分配的id，数据 ]，目前不允许发多段数据
			handler._on_multi_data(msgs)
		else:
			msg = s.recv()
			handler._on_raw_data(msg)

	global hb_time
	if hb_time < time.time():  # 发送心跳包
		hb_time = time.time() + _HEARTBEAT_SEC
		for handler in _sock_list.values():
			handler.send_heartbeat()


def cmd_tick():
	cmd_q = _cmd_queue
	cmd_sock = _zmq_cmd_sock
	try:
		cmd = cmd_q.get(False)
	except Empty:
		return
	if isinstance(cmd, int):  # 退出
		return

	cmd_sock.send(_pack(cmd[0]))

	msg = cmd_sock.recv()
	msg = _unpack(msg)
	if cmd[1]:
		cmd[1](msg)


_mode = 0
MAIN_THREAD_TICK = 0x1
MAIN_THREAD_SOCKET = 0x10
MAIN_THREAD_EVERYTHING = MAIN_THREAD_TICK | MAIN_THREAD_SOCKET
_callback_queue = Queue()


def init(projectID, mode=0, err_handler=None, addr="tcp://59.111.129.112:5560"):
	"""
	初始化SSRPC模块
	:param projectID: 你的项目代号，如G68
	:param mode: 启动模式，目前支持：
	MAIN_THREAD_TICK：由宿主来定时调用tick处理回调；否则将在socket线程自动回调；
	MAIN_THREAD_SOCKET: 由宿主来定时调用socket tick； 否则在子线程进行loop
	:param err_handler: 错误处理函数，格式：err_handler(code, msg)
	:param addr: broker地址，默认可不填写
	"""
	global _zmq_context
	global _zmq_poll
	global _zmq_cmd_sock
	global _cmd_thread
	global _socket_thread
	global _project
	global _mode
	global _err_handler

	assert projectID
	_project = projectID
	_mode = mode
	_err_handler = err_handler

	_zmq_context = zmq.Context()
	_zmq_cmd_sock = _zmq_context.socket(zmq.REQ)
	_zmq_cmd_sock.connect(addr)

	_zmq_poll = zmq.Poller()

	if not mode & MAIN_THREAD_SOCKET:
		_cmd_thread = threading.Thread(target=cmd_loop, args=(_cmd_queue, _zmq_cmd_sock))
		_cmd_thread.daemon = True
		_cmd_thread.start()
		_socket_thread = threading.Thread(target=socket_loop, args=(_zmq_poll,))
		_socket_thread.daemon = True
		# _socket_thread.start()

if not is_py2:
	def _convert_bytes_to_str(data):
		if isinstance(data, bytes):
			return data.decode()
		elif isinstance(data, (tuple, list)):
			return [_convert_bytes_to_str(x) for x in data]
		elif isinstance(data, dict):
			new_dict = {}
			for k, v in data.items():
				new_dict[_convert_bytes_to_str(k)] = _convert_bytes_to_str(v)
			return new_dict
		else:
			return data


def _send_cmd(data, cb=None):
	_cmd_queue.put([data, cb])


def _setup_cb(handler, new_addr):
	global _sock_list
	if not new_addr:
		_err_msg(SSRPC_ERR.CONNECT_FAILED, "try connect to %s failed" % handler._to_server_id)
		return
	new_sock = _zmq_context.socket(zmq.DEALER)
	new_sock.connect(new_addr)
	_zmq_poll.register(new_sock, zmq.POLLIN)
	if not(_mode & MAIN_THREAD_SOCKET) and (not _socket_thread.is_alive()):
		_socket_thread.start()
	_sock_list[new_sock] = handler
	# print(handler)
	handler.set_zmq_socket(new_sock)
	handler.on_init_finish(True)
	print("create", "server" if handler.is_server else "client", "done ->", new_addr)
	if not handler.is_server:
		try:
			import Atmosphere
			Atmosphere.EventParasiteConnected()
		except Exception as e:
			print(e)

SERV_MODE_ALL_GET_MSG = 0x1
def setup_server(server_ids, handler, mode=0):
	if not isinstance(server_ids, (tuple, list)):
		server_ids = [server_ids]
	data = {
		"server_ids": server_ids,
		"cmd": CMD_CREATE_SERVER,
		"proj": _project,
		"mode": mode,
	}

	handler._my_id = server_ids[0]
	_send_cmd(data, lambda msg: _setup_cb(handler, msg["rtn"]))


def setup_client(server_id, handler):
	handler._to_server_id = server_id
	import uuid
	handler._my_id = uuid.uuid4()
	data = {
		"server_id":server_id,
		"cmd":CMD_CREATE_CLIENT,
		"proj":_project,
	}
	_send_cmd(data, lambda msg: _setup_cb(handler, msg["rtn"]))


def get_server_node_info(project_id, callback):
	data = {
		"projectID": project_id,
		"cmd": CMD_GET_SERVER_NODE,
	}
	_send_cmd(data, lambda msg: callback(project_id, msg["rtn"]))


def _recv_node_cb(project_id, rtn):
	print("broker servers(project id: %s):" % project_id)
	for serv, ports in rtn.items():
		print("server id: %s, node ports: %s" % (serv, ports))


def tick():
	if _mode & MAIN_THREAD_TICK:
		while _callback_queue.qsize() > 0:
			cb = _callback_queue.get()
			cb[0]._exec_func(cb[1], cb[2])
	if _mode & MAIN_THREAD_SOCKET:
		socket_tick()
		cmd_tick()


def stop():
	pass
	# global _zmq_poll
	# global _sock_list

	# _cmd_thread.join()
	# _socket_thread.join()

	# _zmq_poll.close()
	# for sock, handler in _sock_list.items():
	# 	sock.close()
	# 	if handler.is_server:
	# 		_send_cmd(
	# 			{
	# 				"cmd":CMD_SERVER_QUIT,
	# 				"server_id":handler._my_id,
	# 				"proj":_project,
	# 			}
	# 		)
	# _sock_list = None

def ssrpc_method(rpcType=0):
	def _wrap_func(func):
		import sys
		classLocals = sys._getframe(1).f_locals
		dictName = "__rpc_methods__"
		if dictName not in classLocals:
			classLocals[dictName] = {"__ssrpc_callback__": lambda x: x}

		name = func.__name__

		assert name not in classLocals[dictName]
		classLocals[dictName][name] = rpcType
		return func
	return _wrap_func

def _ssrpc_class_inherit(clz):

	if not hasattr(clz, "__rpc_methods__") or "__inherited" in clz.__rpc_methods__:
		return
	rpc_dict = {"__inherited":True}
	for x in reversed(clz.mro()):
		if hasattr(x, "__rpc_methods__"):
			rpc_dict.update(x.__rpc_methods__)
	clz.__rpc_methods__ = rpc_dict


class _RPCb(object):
	'''
	SSRPC调用完后的返回对象，用于实现异步/同步(callback/wait)调用
	'''
	def __init__(self, prx):
		super(_RPCb, self).__init__()
		self.proxy = prx
		self.curr_sess_id = None

	def callback(self, func):
		assert self.curr_sess_id
		self.proxy.cb_list[self.curr_sess_id] = func
		self.curr_sess_id = None

	def wait(self, timeout=0):
		assert _mode & MAIN_THREAD_TICK, "wait can only be used in MAIN_THREAD_TICK mode"
		# TODO 加入Timeout
		old_time = (time.time() + timeout) if timeout > 0 else 0
		cache_cmd = []
		got_data = None
		while True:
			if _callback_queue.qsize() > 0:
				cb = _callback_queue.get()
				data = self.proxy.handler._unpack_data(cb[1])
				if data.get("__cb_session_id") == self.curr_sess_id:
					got_data = data
					break
				else:
					cache_cmd.append(cb)

			if old_time > 0 and time.time() > old_time:
				break

			time.sleep(0)

		for cmd in cache_cmd:  # 把cache的压回去
			_callback_queue.put(cmd)

		return got_data["__args"] if got_data else None


class _RPCProxy(object):
	'''
	具体的RPC调用对象，会把rpc封装成协议包发送，及解封接收到的包
	'''
	SUNSHINE_UUID = None

	def __init__(self, handler, is_server=False, reg_all=False):
		super(_RPCProxy, self).__init__()
		print (handler, is_server, reg_all);
		self.handler = handler
		self.is_server = is_server
		if reg_all:
			self.reg_all()
		if self.is_server:
			_ssrpc_class_inherit(self.__class__)
		self.cb_sess_id = None
		self.cb_sess_id_dict_for_server = {}
		self.cb_list = {}

		self.cb_obj = _RPCb(self)

	def __getitem__(self, client_id):
		self.client_id = client_id
		return self

	def __getattr__(self, item):
		def _call(*args, **kwargs):
			kwargs["__args"] = args
			kwargs["__rpc_func"] = item
			kwargs["__session_id"] = uuid.uuid4().hex
			kwargs["__handler_id"] = self.SUNSHINE_UUID
			if self.is_server:
				self.handler.send(self.client_id, kwargs)
				delattr(self, "client_id")
			else:
				self.handler.send(kwargs)

			self.cb_obj.curr_sess_id = kwargs["__session_id"]
			return self.cb_obj

		return _call

	def try_parse_kwargs(self, data):
		if isinstance(data, dict) and "__rpc_func" in data:
			func_name = data.pop("__rpc_func")

			if func_name in getattr(self.__class__, "__rpc_methods__", {}):
				func = getattr(self, func_name, None)
				args = data.pop("__args", [])
				return func, args, data, data.pop("__session_id", None), data.pop("__cb_session_id", None)
			else:
				raise RuntimeError("rpc function %s not found" % (func_name))

		return None, None, data, None, None

	def _exec_func(self, data, cid=None):
		# print("_exec_func", self, data)
		try:
			func, args, data, sid, cb_sid = self.try_parse_kwargs(data)
			if cb_sid:  # this is a callback message
				func = self.cb_list.pop(cb_sid, None)
				if func:
					func(*args, **data)
			elif func:
				# print("func:{}, args:{}, cid:{}, data:{}".format(func, args, cid, data))
				rtn = func(cid, *args, **data) if cid else func(*args, **data)

				if rtn is not None:  # callback message generate
					kwargs = {}
					kwargs["__rpc_func"] = "__ssrpc_callback__"
					kwargs["__args"] = [rtn] if not isinstance(rtn, tuple) else rtn
					kwargs["__cb_session_id"] = sid
					kwargs["__handler_id"] = self.SUNSHINE_UUID
					if self.is_server:
						self.handler.send(cid, kwargs)
					else:
						self.handler.send(kwargs)
			else:
				self.handler.on_data(cid, data) if cid else self.handler.on_data(data)
		except:
			import traceback
			traceback.print_exc()

	def reg_all(self):
		if not hasattr(self.__class__, "__rpc_methods__"):
			setattr(self.__class__, "__rpc_methods__", {})
		for k, func in self.Register(None).items():
			registry = self.__class__.__rpc_methods__
			registry[func.__name__] = func
			# print("reg", func)

	def Register(self, callServer):
		return {}


class ZMQHandler(object):
	_zmq_socket = None
	_my_id = None  # 这个并不是server.on_data里面的client_id
	_to_server_id = None

	def __init__(self):
		super(ZMQHandler, self).__init__()
		self._SSRPC = None
		self._rpc_registry = {}

	@property
	def is_server(self):
		return self._to_server_id is None

	@property
	def is_connected(self):
		return self._zmq_socket is not None

	def set_zmq_socket(self, so):
		self._zmq_socket = so
		self._zmq_cache_msgs = []
		if self.is_server:
			self.send(b"imsvr", self._my_id)

	def stop(self):
		self._zmq_socket = None

	_ZIP_MAGIC_STR = b"\xffsrz"  # zip的包头判定字符串
	_ZIP_MIN_SIZE = 1024  # 自动进行zip的包体大小下限

	def _unpack_data(self, data):
		if len(data) > 4 and data[:4] == self._ZIP_MAGIC_STR:
			import zlib
			data = zlib.decompress(data[4:])
		data = _unpack(data)

		if not is_py2:
			data = _convert_bytes_to_str(data)

		return data

	def _pack_data(self, data):
		data = _pack(data)
		if len(data) > self._ZIP_MIN_SIZE:
			import zlib
			data = self._ZIP_MAGIC_STR + zlib.compress(data)
		return data

	# @property
	# def SSRPC(self):
	# 	if not self._SSRPC:
	# 		self._SSRPC = _RPCProxy(self, self.is_server)
	# 	return self._SSRPC

	def register_rpc_handler(self, rpc_hanlder):
		if rpc_hanlder.SUNSHINE_UUID in self._rpc_registry:
			raise RuntimeError("duplicated SUNSHINE_UUID %s" % rpc_hanlder.SUNSHINE_UUID)
		self._rpc_registry[rpc_hanlder.SUNSHINE_UUID] = rpc_hanlder

	def _exec_func(self, data, cid=None):
		data = self._unpack_data(data)
		rpc_hanlder_id = data.pop("__handler_id")
		try:
			rpc_hanlder = self._rpc_registry[rpc_hanlder_id]
		except KeyError:
			print("SUNSHINE_UUID %s not found" % rpc_hanlder_id)
			return
		if _mode & MAIN_THREAD_TICK and threading.current_thread() is _socket_thread:
			_callback_queue.put([rpc_hanlder, data, cid])
		else:
			rpc_hanlder._exec_func(data, cid)
		# self.SSRPC._exec_func(data, cid)
	
	def __getitem__(self, handlerid):
		return self._rpc_registry[handlerid]


class SSRPCSvrHandler(ZMQHandler):
	def __init__(self):
		super(SSRPCSvrHandler, self).__init__()
		self.online_clients = {}
		self._clientInfoCBFunc = None

	def on_init_finish(self, rtn):
		pass

	def on_data(self, client_id, data):
		raise NotImplementedError()

	def on_client_connect(self, client_id):
		print("client", _hex_str(client_id), "connect")
		pass

	def on_client_disconnect(self, client_id):
		print("client", _hex_str(client_id), "disconnect")
		pass

	def _on_multi_data(self, msgs):
		self._zmq_cache_msgs.extend(msgs)
		while len(self._zmq_cache_msgs) >= 2:
			cid = self._zmq_cache_msgs.pop(0)
			data = self._zmq_cache_msgs.pop(0)
			if cid == b"myclients_cb":  # 特殊信息
				self.all_clients_info(_unpack(data))
				continue
			if cid not in self.online_clients:
				self.on_client_connect(cid)
			self.online_clients[cid] = time.time()

			if data != _HEARTBEAT_DATA:
				self._exec_func(data, cid)

	def send(self, client_id, data):
		self._zmq_socket.send_multipart([client_id, self._pack_data(data)])

	def return_sync_result(self, client_id, data, session_id):
		send_data = {"data": data, "__session_id": session_id}
		self._zmq_socket.send_multipart([client_id, self._pack_data(send_data)])

	def send_heartbeat(self):
		self._zmq_socket.send_multipart([_HEARTBEAT_DATA, _pack(self._my_id)])

		for cli in list(self.online_clients):
			if time.time() - self.online_clients[cli] > _HEARTBEAT_SEC * 2:
				self.online_clients.pop(cli)
				self.on_client_disconnect(cli)

	def get_all_clients(self, cbFunc=None):
		"""获取连接该节点的客户端信息，中心节点上的clients"""
		self.send(b"myclients", "")
		self._clientInfoCBFunc = cbFunc

	def all_clients_info(self, data):
		"""从连接节点返回的客户端信息"""
		if self._clientInfoCBFunc:
			self._clientInfoCBFunc(data)
		else:
			print("all connected clients:")
			for key, val in data.items():
				print("client: %s, info: %s" % (_hex_str(key), val))  # debug用


class SSRPCCliHandler(ZMQHandler):

	def __init__(self):
		self.__callbacks = {}  # callback缓存
		super(SSRPCCliHandler, self).__init__()

	def _registerCallback(self, callback):
		""" 注册回调 """
		callbackId = str(uuid.uuid4()).encode("UTF-8")  # python 2 与 3 的 str 不同，所以传bytes
		self.__callbacks[callbackId] = callback
		return callbackId

	@ssrpc_method()
	def callback(self, callback_id, *args, **kwargs):
		"""Server回调过来，根据 callback_id 找回 callback 函数"""
		callback_id = callback_id.encode("UTF-8")
		self.__callbacks[callback_id](*args, **kwargs)
		del self.__callbacks[callback_id]

	def on_init_finish(self, rtn):
		print("on_init_finish", self, rtn)

	def on_data(self, data):
		raise NotImplementedError()

	def _on_raw_data(self, data):
		self._exec_func(data)

	def send(self, data):
		if hasattr(self, "_zmq_socket") and self._zmq_socket is not None:
			self._zmq_socket.send(self._pack_data(data))

	def send_heartbeat(self):
		self._zmq_socket.send(_HEARTBEAT_DATA)



# coding=utf-8
__author__ = 'lxn3032'


import math


class Vec2(object):
    def __init__(self, x=0.0, y=0.0):
        self.x = x
        self.y = y

    @staticmethod
    def from_radian(rad):
        x = math.cos(rad)
        y = math.sin(rad)
        return Vec2(x, y)

    def __add__(self, other):
        return Vec2(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        return Vec2(self.x - other.x, self.y - other.y)

    def __radd__(self, other):
        return Vec2(other.x + self.x, other.y + self.y)

    def __rsub__(self, other):
        return Vec2(other.x - self.x, other.y - self.y)

    def __mul__(self, other):
        return Vec2(self.x * other, self.y * other)

    def __rmul__(self, other):
        return Vec2(self.x * other, self.y * other)

    def to_list(self):
        return [self.x, self.y]

    @classmethod
    def intersection_angle(cls, v1, v2):
        cosval = cls.dot_product(v1, v2) / (v1.length * v2.length)
        if -2 < cosval < -1:
            cosval = -1
        elif 1 < cosval < 2:
            cosval = 1
        return math.acos(cosval) * (1 if cls.cross_product(v1, v2) > 0 else -1)

    @staticmethod
    def dot_product(v1, v2):
        return v1.x * v2.x + v1.y * v2.y

    @staticmethod
    def cross_product(v1, v2):
        return v1.x * v2.y - v2.x * v1.y

    @property
    def length(self):
        return math.sqrt(self.x ** 2 + self.y ** 2)

    def unit(self):
        length = self.length
        return Vec2(self.x / length, self.y / length)

    def rotate(self, radian):
        qx = math.cos(radian) * self.x - math.sin(radian) * self.y
        qy = math.sin(radian) * self.x + math.cos(radian) * self.y
        self.x, self.y = qx, qy

    def __str__(self):
        return '({}, {})'.format(self.x, self.y)
    __repr__ = __str__



# -*- coding: utf-8 -*-
# @Author: gzliuxin
# @Email:  gzliuxin@corp.netease.com
# @Date:   2017-07-14 19:48:10
from poco.drivers.netease.mh import MhPoco


def set_forground(title):
    from airtest.core.win import Windows
    dev = Windows()
    handles = dev.find_window_list(title)
    if not handles:
        raise RuntimeError("window not found")
    dev.set_handle(handles[0])
    dev.set_foreground()


if __name__ == '__main__':
    # p = MhPoco(addr=("10.254.45.54", 5001))  # android: Meizu
    p = MhPoco(addr=("10.254.245.124", 5001))  # android: Meizu
    # p = MhPoco(addr=("10.254.42.28", 5001))  # ios: ff's iPad
    # p = MhPoco(addr=("10.211.55.1", 5001))  # windows: parallel
    # p = MhPoco()
    # p(text="长安城").click()
    # p(textMatches="^比武场\[.+\]$").click("center")
    # p(text="手机也能玩").click()
    # p(u"超级神羊").click()
    # p(u"超级神羊").focus((0, 0)).click()
    # p(text="超级神虎").sibling(type="Button").click()
    # l = p(type="CBuffPanel").offspring(type="CDisableImage")
    # for i in l:
    #     i.click()
    l = p(type="CPanel").child(type="AttendanceItem").wait(10)
    print(l.exists())

    # 点日历里的每一项
    # l = p(type="CPanel").child(type="AttendanceItem")
    # for i in l:
    #     i.click("center")

    # while True:
    #     p('loginext/进入互通幻境').click()
    #     p('loginext/确定按钮').focus([0.5, -0.5]).click()
    #     p('loginext/确定按钮').click()


    # for n in p():
    #     print n.get_text()

    # import random
    # import time

    # d = p(type="VDialog")
    # while True:
    #     l = p(type="Button")
    #     r = random.randint(0, len(l.nodes))
    #     try:
    #         i = l[r]
    #     except:
    #         continue
    #     i.click()

    # # 测试截图接口
    # p._rpc_client.getPortSize()
    # from base64 import b64decode
    # d = p._rpc_client.get_screen()
    # with open("screen.png", "wb") as f:
    #     f.write(b64decode(d))



# coding=utf-8

from setuptools import setup, find_packages
from pip.req import parse_requirements

import poco


# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements('requirements.txt', session=False)

# reqs is a list of requirement
reqs = [str(ir.req) for ir in install_reqs if ir.req]

setup(
    name='pocoui',
    version=poco.__version__,
    keywords="poco",
    description='Poco cross-engine UI automated test framework.',
    packages=find_packages(),
    include_package_data=True,
    install_requires=reqs,
    license='Apache License 2.0',

    author='adolli, adolli, lxn3032, gzliuxin',
    author_email='adolli@163.com, adollixiang@gmail.com, lxn3032@corp.netease.com, gzliuxin@corp.netease.com',
    url='https://github.com/AirtestProject/Poco',
)



# coding=utf-8


import time
import base64
import unittest

from poco.drivers.android.uiautomation import AndroidUiautomationPoco
from airtest.core.android import Android


class TestCommonCases(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.poco = AndroidUiautomationPoco()

    def test_snapshot(self):
        b64img = self.poco.snapshot()
        with open('img.jpg', 'wb') as img:
            img.write(base64.b64decode(b64img))

    def test_set_text(self):
        self.poco('com.netease.my:id/netease_mpay__login_urs').set_text('adolli@163.com')

    def test_existence(self):
        self.assertTrue(self.poco('登录').exists())

    def test_existence_after_hide(self):
        btn = self.poco(text='快速游戏', type='android.widget.TextView').focus([0.5, -3])
        btn.click()
        self.assertEqual(btn.exists(), True)
        btn.invalidate()
        self.assertEqual(btn.exists(), False)
        time.sleep(1)
        btn2 = self.poco(text='快速游戏', type='android.widget.Button')
        self.assertEqual(btn2.exists(), True)

    def test_any(self):
        btn = self.poco(text='快速游戏', type='android.widget.TextView').focus([0.5, -3])
        btn.click()
        time.sleep(1)
        print btn.exists()

    def test_dump(self):
        hierarchy = self.poco.agent.hierarchy.dump()
        print hierarchy


class TestRemoteDevice(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.device = Android('a6c29d2b', adbhost=('10.250.190.230', 5038))
        cls.poco = AndroidUiautomationPoco(cls.device)

    def test_any(self):
        for n in self.poco():
            print n.get_name()




#!/usr/bin/env
# -*- coding: utf-8 -*-
"""parse apk file class, inherit from axmlparserpy."""
import os
from pydash import _


class APKParse(object):
    """APK parse."""

    def __init__(self, filename):
        """Init method."""
        super(APKParse, self).__init__()
        self.filename = filename
        self.valid_zip = False

        parse_data = self.get_aapt_parse_data('package')
        self.parse_dict = dict(item.split('=') for item in _.head(parse_data).split(' '))

    @property
    def package(self):
        package_name = self.parse_dict.get('name')
        if not package_name:
            raise RuntimeError('Cannot get package name from "{}"'.format(self.filename))
        return package_name

    @property
    def versionCode(self):
        vc = self.parse_dict.get('versionCode')
        if not vc:
            raise RuntimeError('Cannot get versionCode from "{}"'.format(self.filename))
        return int(vc)

    def get_raw(self):
        """Get raw data."""
        with open(self.filename, 'rb') as f:
            return f.read()

    def get_aapt_parse_data(self, app_type):
        """Get aapt parse data."""
        aapt_line = "aapt dump badging %s | grep '%s' | awk -F ':' \
            '{print $2}'" % (self.filename, app_type)
        aapt_data = os.popen(aapt_line).read().splitlines()
        parse_data = [d.replace("'", '').strip() for d in aapt_data if d]
        return parse_data

    def parse_package_version(self):
        """Parse package and version from apk."""
        parse_data = self.get_aapt_parse_data('package')
        package, version = '', ''
        if not parse_data:
            return package, version
        parse_dict = dict(
            item.split('=') for item in _.head(parse_data).split(' '))
        package = parse_dict.get('name', '')
        version = parse_dict.get('versionName', '')
        return package, version

    def parse_app_name(self):
        """Parse app name form apk."""
        parse_data = self.get_aapt_parse_data('application-label')
        app_name = _.head(parse_data) if parse_data else ''
        return app_name.decode("utf8")



# coding=utf-8

__author__ = 'lxn3032'
__all__ = ['Attributor']


class Attributor(object):
    """
    This is a helper class to access the node attributes. In some cases it is not possible to explicitly invoke the
    node member functions thus the following two functions are introduced.

    The instance of this class will be used in implementation of :py:class:`HierarchyInterface \
    <poco.sdk.interfaces.hierarchy.HierarchyInterface>`.

    .. note:: Do not call these methods explicitly in the test codes.
    """

    def getAttr(self, node, attrName):
        if type(node) in (list, tuple):
            node = node[0]
        return node.getAttr(attrName)

    def setAttr(self, node, attrName, attrVal):
        if type(node) in (list, tuple):
            node = node[0]
        node.setAttr(attrName, attrVal)



# coding=utf-8

import re
import json
import sys
import traceback

from poco.drivers.unity3d import UnityPoco
from airtest.core.api import connect_device


def process_cmd(cmd):
    try:
        print('cmd', cmd)
        exec(cmd)
    except:
        traceback.print_exc()


if __name__ == '__main__':
    print('hello poco unity3d repl!')

    buf = ''
    while True:
        line = sys.stdin.readline().rstrip() + '\n'
        buf += line
        if re.findall(r'# end-proc #\s*$', buf):
            try:
                # print('cmd', buf)
                cmd = buf
                buf = ''
                exec(cmd)
            except Exception as e:
                traceback.print_exc()



SECRET_KEY = 'secret'
ROOT_URLCONF = 'jsonrpc.tests.test_backend_django.urls'
ALLOWED_HOSTS = ['testserver']
DATABASE_ENGINE = 'django.db.backends.sqlite3'
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}
